import sys
import inspect
from iapws import IAPWS97, IAPWS95
from yangke.performance.iapws97 import get_p_by_th, get_h_by_pt, get_t_by_hp


class TestPoint:
    """
    机组测点的流量及热力参数
    """

    def __init__(self):
        self.p: float = -1
        self.t: float = -1
        self.h: float = -1
        self.mass_flow = -1  # 质量流量
        self.available: bool = False  # 参数是否可用，仅当所有参数都可用时为True
        self.necessary: bool = True  # 参数是否必要

    def init_state(self):
        pass

    def set_state_by_name_and_value_list(self, test_point_name_list, value_list):
        """
        根据试验测点中已有数据更新测点中水或水蒸汽气的状态数据

        :param test_point_name_list:
        :param value_list:
        :return:
        """
        t_n = 0
        p_n = 0
        t = 0
        p = 0
        for name, value in zip(test_point_name_list, value_list):
            if "温度" in name:
                t = (value + t_n * t) / (t_n + 1)
                t_n = t_n + 1
            elif "压力" in name:
                p = (value + p_n * p) / (p_n + 1)
                p_n = p_n + 1
            elif "流量" in name:
                self.mass_flow = value
        if t != 0:
            self.t = t
        if p != 0:
            self.p = p
        if self.p != -1 and self.t != -1:
            self.available = True
            self.h = get_h_by_pt(p, t + 273.15)


class Case:
    """
    定义单个试验工况的数据组合，用于完成热耗、缸效率等的计算
    """
    parameters_available = {}  # 可用参数字典
    time_period = {}  # 性能试验数据采集时间段
    main_steam: "主蒸汽,主汽门前" = TestPoint()  # 不区分是左侧还是右侧主汽门
    adjust_steam: "调节级" = TestPoint()
    outlet_high: "高压缸排汽" = TestPoint()
    inlet_medium: "再热汽门前,热再,再热蒸汽" = TestPoint()
    outlet_medium: "中排,中压缸排汽" = TestPoint()
    inlet_low: "低压缸进汽" = TestPoint()
    outlet_low: "低压缸排汽" = TestPoint()
    outlet_turbine_pump: "小机排汽" = TestPoint()
    extract_1: "一段抽汽,一抽汽" = TestPoint()
    heater_1_vapor_in: "一号高加进汽,一高加进汽" = TestPoint()
    heater_1_vapor_out: "一号高加疏水,一高加疏水" = TestPoint()
    heater_1_water_in: "一号高加进水,一高加进水" = TestPoint()
    heater_1_water_out: "一号高加出水,一高加出水" = TestPoint()
    extract_2: "二段抽汽,二抽汽" = TestPoint()
    heater_2_vapor_in: "二号高加进汽,二高加进汽" = TestPoint()
    heater_2_vapor_out: "二号高加疏水,二高加疏水" = TestPoint()
    heater_2_water_in: "二号高加进水,二高加进水" = TestPoint()
    heater_2_water_out: "二号高加出水,二高加出水" = TestPoint()
    extract_3: "三段抽汽,三抽汽" = TestPoint()
    heater_pre_3_vapor_in: "蒸冷器进汽" = TestPoint()
    heater_pre_3_water_out: "蒸冷器出水" = TestPoint()
    heater_3_vapor_in: "三号高加进汽,三高加进汽" = TestPoint()
    heater_3_vapor_out: "三号高加疏水,三高加疏水" = TestPoint()
    heater_3_water_in: "三号高加进水,三高加进水" = TestPoint()
    heater_3_water_out: "三号高加出水,三高加出水" = TestPoint()
    extract_4: "四段抽汽,四抽汽" = TestPoint()
    heater_4_vapor_in: "四号高加进汽,四高加进汽" = TestPoint()
    heater_4_vapor_out: "四号高加疏水,四高加疏水" = TestPoint()
    heater_4_water_in: "四号高加进水,四高加进水" = TestPoint()
    heater_4_water_out: "四号高加出水,四高加出水" = TestPoint()
    extract_5: "五段抽汽,五抽汽" = TestPoint()
    heater_5_vapor_in: "五号低加进汽,五低加进汽" = TestPoint()
    heater_5_vapor_out: "五号低加疏水,五低加疏水" = TestPoint()
    heater_5_water_in: "五号低加进水,五低加进水" = TestPoint()
    heater_5_water_out: "五号低加出水,五低加出水" = TestPoint()
    extract_6: "六段抽汽,六抽汽" = TestPoint()
    heater_6_vapor_in: "六号低加进汽,六低加进汽" = TestPoint()
    heater_6_vapor_out: "六号低加疏水,六低加疏水" = TestPoint()
    heater_6_water_in: "六号低加进水,六低加进水" = TestPoint()
    heater_6_water_out: "六号低加出水,六低加出水" = TestPoint()
    extract_7: "七段抽汽,七抽汽" = TestPoint()
    heater_7_vapor_in: "七号低加进汽,七低加进汽" = TestPoint()
    heater_7_vapor_out: "七号低加疏水,七低加疏水" = TestPoint()
    heater_7_water_in: "七号低加进水,七低加进水" = TestPoint()
    heater_7_water_out: "七号低加出水,七低加出水" = TestPoint()
    extract_8: "八段抽汽,八抽汽" = TestPoint()
    heater_8_vapor_in: "八号低加进汽,八低加进汽" = TestPoint()
    heater_8A_vapor_in: "八A加热器进汽" = TestPoint()
    heater_8B_vapor_in: "八B加热器进汽" = TestPoint()
    heater_8_vapor_out: "八号低加疏水,八低加疏水" = TestPoint()
    heater_8A_vapor_out: "八A号低加疏水,八A低加疏水" = TestPoint()
    heater_8B_vapor_out: "八B号低加疏水,八B低加疏水" = TestPoint()
    heater_8_water_in: "八号低加进水,八低加进水" = TestPoint()
    heater_8A_water_in: "八A号低加进水,八B低加进水" = TestPoint()
    heater_8B_water_in: "八A号低加进水,八B低加进水" = TestPoint()
    heater_8_water_out: "八号低加出水,八低加出水" = TestPoint()
    heater_8A_water_out: "八A号低加出水,八A低加出水" = TestPoint()
    heater_8B_water_out: "八B号低加出水,八B低加出水" = TestPoint()
    heater_9A_vapor_in: "九A加热器进汽,九A号加热器进汽" = TestPoint()
    heater_9B_vapor_in: "九B加热器进汽,九A号加热器进汽" = TestPoint()
    heater_9A_vapor_out: "九A加热器疏水,九A号加热器疏水" = TestPoint()
    heater_9B_vapor_out: "九B加热器疏水,九B号加热器疏水" = TestPoint()
    heater_9A_water_in: "九A加热器进水,九A号加热器进水" = TestPoint()
    heater_9B_water_in: "九B加热器进水,九B号加热器进水" = TestPoint()
    heater_9A_water_out: "九A加热器出水,九A号加热器出水" = TestPoint()
    heater_9B_water_out: "九B加热器出水,九B号加热器出水" = TestPoint()
    seal_heater_1_water_in: "一号轴加进水,轴加进水" = TestPoint()
    seal_heater_2_water_in: "二号轴加进水,二轴加进水" = TestPoint()  # 包含二号轴加进水，排除轴加进水
    seal_heater_vapor_in: "汽封加热器进汽,轴封加热器进汽" = TestPoint()
    # seal_heater_vapor_out: "汽封加热器"
    seal_heater_water_in: "汽封加热器进水,轴封加热器进水,轴加进水" = TestPoint()
    seal_heater_water_out: "汽封加热器出水,轴封加热器出水,轴加出水" = TestPoint()
    # ------------------------------ 热井至加热器之间测点 ----------------------------
    heat_well_out: "热井出" = TestPoint()
    water_condense_pump_in: "凝泵进" = TestPoint()
    water_condense_pump_A_in: "凝泵A进" = TestPoint()
    water_condense_pump_B_in: "凝泵B进" = TestPoint()
    water_condense_pump_out: "凝泵出" = TestPoint()
    water_condense_pump_A_out: "凝泵A出" = TestPoint()
    water_condense_pump_B_out: "凝泵B出" = TestPoint()
    # ------------------------------ 低加出口至高加进口之间测点 ----------------------------
    main_condense_water: "主凝结水,进除氧器凝结水,除氧器进水,除氧器进口凝结水" = TestPoint()
    deaerator_vapor_in: "除氧器进汽" = TestPoint()
    deaerator_drain_water_in: "进除氧器疏水" = TestPoint()
    deaerator_out: "除氧器出水,除氧器下水" = TestPoint()
    pump_turbine_vapor_in: "小机进汽,给水泵汽轮机进汽" = TestPoint()
    pump_before_water_in: "前置泵进" = TestPoint()
    pump_before_water_out: "前置泵出" = TestPoint()
    feed_pump_in: "给水泵进,汽泵进" = TestPoint()
    feed_pump_out: "给水泵出,汽泵出" = TestPoint()
    final_feed_water: "最终给水" = TestPoint()
    water_reheater_reducing: "再热减温水,再减,再热器减温水" = TestPoint()
    water_overheat_reducing_1: "过热一级减温水" = TestPoint()
    water_overheat_reducing_2: "过热二级减温水" = TestPoint()
    p_env: "大气压力" = 0
    condenser_water_level: "凝汽器水位" = 0
    deaerator_water_level: "除氧器水位" = 0
    # ------------------------------------ 省煤器 ----------------------------------------
    economizer_water_outlet: "省煤器出,省煤器回" = TestPoint()  # 省煤器和低温省煤器的测点都会归到该勒种
    economizer_water_inlet: "省煤器进水" = TestPoint()
    # ------------------------------------ 漏气 ----------------------------------------
    leakage_hp_seal_main: "高压缸前.后轴封漏汽母管" = TestPoint()
    leakage_hp_to_extract_3: "高压门杆漏气至三抽" = TestPoint()
    leakage_hp_to_ssr: "高压门杆漏气至SSR" = TestPoint()
    leakage_hp_seal_to_extract_4: "高压缸前.后轴封漏汽至四" = TestPoint()
    leakage_tail_ip_seal_to_ssr: "中压缸后轴封二段漏气至SS,中压后轴封二漏至SS,中压后轴封二漏段漏汽至SS" = TestPoint()
    leakage_head_ip_seal_to_ssr: "中压缸前轴封二段漏气至SS" = TestPoint()
    # ------------------------------------ 循环水 ----------------------------------------
    condenser_recycled_water_in: "凝汽器循环水进水" = TestPoint()
    condenser_A_recycled_water_in: "凝汽器A侧循环水进水" = TestPoint()
    condenser_B_recycled_water_in: "凝汽器B侧循环水进水" = TestPoint()
    condenser_recycled_water_out: "凝汽器循环水出水" = TestPoint()
    condenser_A_recycled_water_out: "凝汽器A侧循环水出水" = TestPoint()
    condenser_B_recycled_water_out: "凝汽器B侧循环水出水" = TestPoint()

    def set_value_of_test_point(self, var_str, test_point_name_list, value_list):
        """
        根据计算程序参数对应的 所有测点的名称和值 给计算程序中测点水/水蒸气赋状态

        :param var_str:
        :param test_point_name_list:
        :param value_list:
        :return:
        """
        obj = getattr(self, var_str)
        if isinstance(obj, TestPoint):
            obj.set_state_by_name_and_value_list(test_point_name_list, value_list)
        else:
            setattr(self, var_str, float(value_list))


# power_station_annotations = sys.modules[__name__].__annotations__
alternative_symbol = {"1": "一", "2": "二", "3": "三", "4": "四", "5": "五", "6": "六", "7": "七",
                      "8": "八", "9": "九", "汽": "气", "0": "零", "零": "〇", "进": "入", "一轴加": "一号轴加", "A": "a",
                      "B": "b", "高加": "加热器", "低加": "加热器", "凝泵": "凝结水泵"}

# 数据采集系统中为绝对压力的测点
pressure_absolute = ["低排", "低压缸排汽", "小机排汽", "七段抽汽", "八段抽汽", "高压汽源至汽封蒸汽冷却器进汽压力",
                     "大气压", "环境压力", "七号低加", "7号低加", "8A低加", "8B低加", "9A低加", "9B低加",
                     # "高压缸前后轴封", "中压缸前轴封", "中压缸后轴封","轴封加热器进汽压力", "高压缸前后轴封漏汽母管",
                     ]

# 数据采集系统中为相对压力的测点
pressure_relative = ["主汽门", "调节级", "高压缸排汽", "再热蒸汽", "再热汽门", "中压缸进汽", "中压缸排汽",
                     "低压缸进汽", "一段抽汽", "二段抽汽",
                     "三段抽汽", "四段抽汽", "五段抽汽",
                     "一抽", "二抽", "三抽", "四抽", "五抽",
                     "高加进汽", "高加", "5号低加", "小机进汽",
                     "除氧器进汽",
                     "最终给水", "给水泵出水", "凝泵", "再热减温水", "高压门杆漏汽",
                     "蒸汽冷却器",
                     "低温省煤器进水压力", "低温省煤器",
                     "蒸冷器", "低省", "凝结水", "前置泵", "汽泵", "循环水"]



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'MainWindow.ui'
#
# Created by: PyQt5 UI code generator 5.15.7
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from yangke.common.QtImporter import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.centralwidget)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.splitter = QtWidgets.QSplitter(self.centralwidget)
        self.splitter.setOrientation(QtCore.Qt.Horizontal)
        self.splitter.setObjectName("splitter")
        self.content_tab = QtWidgets.QTabWidget(self.splitter)
        self.content_tab.setToolTip("")
        self.content_tab.setElideMode(QtCore.Qt.ElideNone)
        self.content_tab.setDocumentMode(False)
        self.content_tab.setTabsClosable(True)
        self.content_tab.setMovable(True)
        self.content_tab.setTabBarAutoHide(False)
        self.content_tab.setObjectName("content_tab")
        self.verticalLayout_5.addWidget(self.splitter)
        MainWindow.setCentralWidget(self.centralwidget)
        self.dockWidget_ATW = QtWidgets.QDockWidget(MainWindow)
        self.dockWidget_ATW.setObjectName("dockWidget_ATW")
        self.dockWidgetContents_3 = QtWidgets.QWidget()
        self.dockWidgetContents_3.setObjectName("dockWidgetContents_3")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.dockWidgetContents_3)
        self.verticalLayout_2.setContentsMargins(1, 1, 1, 1)
        self.verticalLayout_2.setSpacing(1)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.splitter_3 = QtWidgets.QSplitter(self.dockWidgetContents_3)
        self.splitter_3.setOrientation(QtCore.Qt.Horizontal)
        self.splitter_3.setObjectName("splitter_3")
        self.ActiveToolWindow = QtWidgets.QTabWidget(self.splitter_3)
        self.ActiveToolWindow.setTabPosition(QtWidgets.QTabWidget.South)
        self.ActiveToolWindow.setObjectName("ActiveToolWindow")
        self.Terminal1 = QtWidgets.QWidget()
        self.Terminal1.setObjectName("Terminal1")
        self.verticalLayout_11 = QtWidgets.QVBoxLayout(self.Terminal1)
        self.verticalLayout_11.setContentsMargins(1, 1, 1, 1)
        self.verticalLayout_11.setSpacing(1)
        self.verticalLayout_11.setObjectName("verticalLayout_11")
        self.splitter_2 = QtWidgets.QSplitter(self.Terminal1)
        self.splitter_2.setOrientation(QtCore.Qt.Horizontal)
        self.splitter_2.setObjectName("splitter_2")
        self.Terminal = QtWidgets.QWidget(self.splitter_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.Terminal.sizePolicy().hasHeightForWidth())
        self.Terminal.setSizePolicy(sizePolicy)
        self.Terminal.setMinimumSize(QtCore.QSize(200, 20))
        self.Terminal.setStyleSheet("background-color: rgb(85, 255, 255);")
        self.Terminal.setObjectName("Terminal")
        self.TerminalVar = QtWidgets.QWidget(self.splitter_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.TerminalVar.sizePolicy().hasHeightForWidth())
        self.TerminalVar.setSizePolicy(sizePolicy)
        self.TerminalVar.setMinimumSize(QtCore.QSize(200, 20))
        self.TerminalVar.setStyleSheet("background-color: rgb(255, 170, 255);")
        self.TerminalVar.setObjectName("TerminalVar")
        self.verticalLayout_12 = QtWidgets.QVBoxLayout(self.TerminalVar)
        self.verticalLayout_12.setContentsMargins(1, 1, 1, 1)
        self.verticalLayout_12.setSpacing(1)
        self.verticalLayout_12.setObjectName("verticalLayout_12")
        self.verticalLayout_11.addWidget(self.splitter_2)
        self.ActiveToolWindow.addTab(self.Terminal1, "")
        self.Log = QtWidgets.QWidget()
        self.Log.setObjectName("Log")
        self.ActiveToolWindow.addTab(self.Log, "")
        self.verticalLayout_2.addWidget(self.splitter_3)
        self.dockWidget_ATW.setWidget(self.dockWidgetContents_3)
        MainWindow.addDockWidget(QtCore.Qt.DockWidgetArea(8), self.dockWidget_ATW)
        self.dockWidget_4 = QtWidgets.QDockWidget(MainWindow)
        self.dockWidget_4.setFeatures(QtWidgets.QDockWidget.DockWidgetClosable|QtWidgets.QDockWidget.DockWidgetFloatable|QtWidgets.QDockWidget.DockWidgetMovable)
        self.dockWidget_4.setObjectName("dockWidget_4")
        self.dockWidgetContents_7 = QtWidgets.QWidget()
        self.dockWidgetContents_7.setObjectName("dockWidgetContents_7")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.dockWidgetContents_7)
        self.verticalLayout_4.setContentsMargins(1, 1, 1, 1)
        self.verticalLayout_4.setSpacing(1)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.navi_tab = QtWidgets.QTabWidget(self.dockWidgetContents_7)
        self.navi_tab.setObjectName("navi_tab")
        self.component_tab = QtWidgets.QWidget()
        self.component_tab.setObjectName("component_tab")
        self.verticalLayout_7 = QtWidgets.QVBoxLayout(self.component_tab)
        self.verticalLayout_7.setContentsMargins(1, 1, 1, 1)
        self.verticalLayout_7.setSpacing(1)
        self.verticalLayout_7.setObjectName("verticalLayout_7")
        self.lv_com = QtWidgets.QListView(self.component_tab)
        self.lv_com.setObjectName("lv_com")
        self.verticalLayout_7.addWidget(self.lv_com)
        self.navi_tab.addTab(self.component_tab, "")
        self.file_tab = QtWidgets.QWidget()
        self.file_tab.setObjectName("file_tab")
        self.verticalLayout_9 = QtWidgets.QVBoxLayout(self.file_tab)
        self.verticalLayout_9.setContentsMargins(1, 1, 1, 1)
        self.verticalLayout_9.setSpacing(1)
        self.verticalLayout_9.setObjectName("verticalLayout_9")
        self.tv_file = QtWidgets.QTreeView(self.file_tab)
        self.tv_file.setObjectName("tv_file")
        self.verticalLayout_9.addWidget(self.tv_file)
        self.navi_tab.addTab(self.file_tab, "")
        self.scene_tab = QtWidgets.QWidget()
        self.scene_tab.setObjectName("scene_tab")
        self.verticalLayout_8 = QtWidgets.QVBoxLayout(self.scene_tab)
        self.verticalLayout_8.setContentsMargins(1, 1, 1, 1)
        self.verticalLayout_8.setSpacing(1)
        self.verticalLayout_8.setObjectName("verticalLayout_8")
        self.lv_scene = QtWidgets.QListView(self.scene_tab)
        self.lv_scene.setObjectName("lv_scene")
        self.verticalLayout_8.addWidget(self.lv_scene)
        self.navi_tab.addTab(self.scene_tab, "")
        self.verticalLayout_4.addWidget(self.navi_tab)
        self.dockWidget_4.setWidget(self.dockWidgetContents_7)
        MainWindow.addDockWidget(QtCore.Qt.DockWidgetArea(1), self.dockWidget_4)
        self.dockWidget_5 = QtWidgets.QDockWidget(MainWindow)
        self.dockWidget_5.setMinimumSize(QtCore.QSize(300, 400))
        self.dockWidget_5.setObjectName("dockWidget_5")
        self.dockWidgetContents_8 = QtWidgets.QWidget()
        self.dockWidgetContents_8.setObjectName("dockWidgetContents_8")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.dockWidgetContents_8)
        self.verticalLayout_3.setContentsMargins(1, 1, 1, 1)
        self.verticalLayout_3.setSpacing(1)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setSpacing(1)
        self.verticalLayout.setObjectName("verticalLayout")
        self.detail_tab = QtWidgets.QTabWidget(self.dockWidgetContents_8)
        self.detail_tab.setObjectName("detail_tab")
        self.layout_detail_tab = QtWidgets.QWidget()
        self.layout_detail_tab.setObjectName("layout_detail_tab")
        self.verticalLayout_6 = QtWidgets.QVBoxLayout(self.layout_detail_tab)
        self.verticalLayout_6.setContentsMargins(1, 1, 1, 1)
        self.verticalLayout_6.setSpacing(1)
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.info_tab = QtWidgets.QTabWidget(self.layout_detail_tab)
        self.info_tab.setStyleSheet("")
        self.info_tab.setTabPosition(QtWidgets.QTabWidget.South)
        self.info_tab.setObjectName("info_tab")
        self.sfv1_2 = QtWidgets.QWidget()
        self.sfv1_2.setObjectName("sfv1_2")
        self.verticalLayout_10 = QtWidgets.QVBoxLayout(self.sfv1_2)
        self.verticalLayout_10.setContentsMargins(1, 1, 1, 1)
        self.verticalLayout_10.setSpacing(1)
        self.verticalLayout_10.setObjectName("verticalLayout_10")
        self.scrollArea = QtWidgets.QScrollArea(self.sfv1_2)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.sfv = QtWidgets.QWidget()
        self.sfv.setGeometry(QtCore.QRect(0, 0, 278, 16))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.sfv.sizePolicy().hasHeightForWidth())
        self.sfv.setSizePolicy(sizePolicy)
        self.sfv.setObjectName("sfv")
        self.scrollArea.setWidget(self.sfv)
        self.verticalLayout_10.addWidget(self.scrollArea)
        self.info_tab.addTab(self.sfv1_2, "")
        self.result = QtWidgets.QWidget()
        self.result.setObjectName("result")
        self.verticalLayout_13 = QtWidgets.QVBoxLayout(self.result)
        self.verticalLayout_13.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_13.setSpacing(0)
        self.verticalLayout_13.setObjectName("verticalLayout_13")
        self.scrollArea_2 = QtWidgets.QScrollArea(self.result)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.scrollArea_2.sizePolicy().hasHeightForWidth())
        self.scrollArea_2.setSizePolicy(sizePolicy)
        self.scrollArea_2.setWidgetResizable(True)
        self.scrollArea_2.setObjectName("scrollArea_2")
        self.res = QtWidgets.QWidget()
        self.res.setGeometry(QtCore.QRect(0, 0, 280, 16))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.res.sizePolicy().hasHeightForWidth())
        self.res.setSizePolicy(sizePolicy)
        self.res.setObjectName("res")
        self.scrollArea_2.setWidget(self.res)
        self.verticalLayout_13.addWidget(self.scrollArea_2)
        self.info_tab.addTab(self.result, "")
        self.material = QtWidgets.QWidget()
        self.material.setObjectName("material")
        self.verticalLayout_14 = QtWidgets.QVBoxLayout(self.material)
        self.verticalLayout_14.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_14.setSpacing(0)
        self.verticalLayout_14.setObjectName("verticalLayout_14")
        self.scrollArea_3 = QtWidgets.QScrollArea(self.material)
        self.scrollArea_3.setWidgetResizable(True)
        self.scrollArea_3.setObjectName("scrollArea_3")
        self.mat = QtWidgets.QWidget()
        self.mat.setGeometry(QtCore.QRect(0, 0, 280, 16))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.mat.sizePolicy().hasHeightForWidth())
        self.mat.setSizePolicy(sizePolicy)
        self.mat.setObjectName("mat")
        self.scrollArea_3.setWidget(self.mat)
        self.verticalLayout_14.addWidget(self.scrollArea_3)
        self.info_tab.addTab(self.material, "")
        self.verticalLayout_6.addWidget(self.info_tab)
        self.detail_tab.addTab(self.layout_detail_tab, "")
        self.verticalLayout.addWidget(self.detail_tab)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.btn_set_value = QtWidgets.QPushButton(self.dockWidgetContents_8)
        self.btn_set_value.setObjectName("btn_set_value")
        self.horizontalLayout.addWidget(self.btn_set_value)
        self.btn_cancel = QtWidgets.QPushButton(self.dockWidgetContents_8)
        self.btn_cancel.setObjectName("btn_cancel")
        self.horizontalLayout.addWidget(self.btn_cancel)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.verticalLayout_3.addLayout(self.verticalLayout)
        self.dockWidget_5.setWidget(self.dockWidgetContents_8)
        MainWindow.addDockWidget(QtCore.Qt.DockWidgetArea(1), self.dockWidget_5)

        self.retranslateUi(MainWindow)
        self.content_tab.setCurrentIndex(-1)
        self.ActiveToolWindow.setCurrentIndex(0)
        self.navi_tab.setCurrentIndex(0)
        self.detail_tab.setCurrentIndex(0)
        self.info_tab.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

        self.content_tab.tabCloseRequested

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "MainWindow"))
        self.ActiveToolWindow.setTabText(self.ActiveToolWindow.indexOf(self.Terminal1), _translate("MainWindow", "终端"))
        self.ActiveToolWindow.setTabText(self.ActiveToolWindow.indexOf(self.Log), _translate("MainWindow", "日志"))
        self.navi_tab.setTabText(self.navi_tab.indexOf(self.component_tab), _translate("MainWindow", "组件"))
        self.navi_tab.setTabText(self.navi_tab.indexOf(self.file_tab), _translate("MainWindow", "目录"))
        self.navi_tab.setTabText(self.navi_tab.indexOf(self.scene_tab), _translate("MainWindow", "场景"))
        self.info_tab.setTabText(self.info_tab.indexOf(self.sfv1_2), _translate("MainWindow", "规格值"))
        self.info_tab.setTabText(self.info_tab.indexOf(self.result), _translate("MainWindow", "结果"))
        self.info_tab.setTabText(self.info_tab.indexOf(self.material), _translate("MainWindow", "材料"))
        self.detail_tab.setTabText(self.detail_tab.indexOf(self.layout_detail_tab), _translate("MainWindow", "组件信息"))
        self.btn_set_value.setText(_translate("MainWindow", "确定"))
        self.btn_cancel.setText(_translate("MainWindow", "取消"))



import datetime
import os.path
from enum import unique, Enum
from shutil import copyfile

from yangke.base import get_key_value, yield_every_day
from yangke.common.fileOperate import fill_excel_cell
from yangke.sis.Report import Report
from yangke.sis.export_history_values import get_history


@unique
@get_key_value
class TagsRead(Enum):
    发电机功率1 = "DCS1:LOAD"
    发电机功率2 = "DCS1:LOAD"
    发电机功率3 = "DCS1:LOAD"
    供电功率1 = ""
    供电功率2 = ""
    供电功率3 = ""
    燃料消耗量1 = ""  # Nm3/h
    燃料消耗量2 = ""
    燃料消耗量3 = ""
    供热负荷1 = ""  # GW
    供热负荷2 = ""  # GW
    供热负荷3 = ""  # GW
    运行小时数1 = ""
    运行小时数2 = ""
    运行小时数3 = ""


report: Report | None = None


def init_carbon():
    global report
    template_file = os.path.join(os.path.dirname(__file__), "../../web/static/碳排放量计算.xlsx")
    report = Report(template=template_file, tags=TagsRead)
    report.add_fill_rules(
        excel_cell={"sheet": "附表 C.3 化石燃料燃烧排放表", "cell": "E3", },
        content={"expression": "累积值(燃料消耗量)/3600", "date_range": "{year}年1月"}  #
    )
    report.add_fill_rules(
        excel_cell={"sheet": "附表 C.3 化石燃料燃烧排放表", "cell": "F3", },
        content={"expression": "累积值(燃料消耗量)/3600", "date_range": "{year}-2"}  # 填入d
    )
    report.add_fill_rules(
        excel_cell={"sheet": "附表 C.3 化石燃料燃烧排放表", "cell": "G3", },
        content={"expression": "累积值(燃料消耗量)/3600", "date_range": "{year}-3"}  # 填入d
    )
    report.add_fill_rules(
        excel_cell={"sheet": "附表 C.3 化石燃料燃烧排放表", "cell": "I3", },
        content={"expression": "累积值(燃料消耗量)/3600", "date_range": "{year}-4"}  # 填入d
    )
    report.add_fill_rules(
        excel_cell={"sheet": "附表 C.3 化石燃料燃烧排放表", "cell": "J3", },
        content={"expression": "累积值(燃料消耗量)/3600", "date_range": "{year}-5"}  # 填入d
    )
    report.add_fill_rules(
        excel_cell={"sheet": "附表 C.3 化石燃料燃烧排放表", "cell": "K3", },
        content={"expression": "累积值(燃料消耗量)/3600", "date_range": "{year}-6"}  # 填入d
    )
    report.add_fill_rules(
        excel_cell={"sheet": "附表 C.3 化石燃料燃烧排放表", "cell": "M3", },
        content={"expression": "累积值(燃料消耗量)/3600", "date_range": "{year}-7"}  # 填入d
    )
    report.add_fill_rules(
        excel_cell={"sheet": "附表 C.3 化石燃料燃烧排放表", "cell": "N3", },
        content={"expression": "累积值(燃料消耗量)/3600", "date_range": "{year}-8"}  # 填入d
    )
    report.add_fill_rules(
        excel_cell={"sheet": "附表 C.3 化石燃料燃烧排放表", "cell": "O3", },
        content={"expression": "累积值(燃料消耗量)/3600", "date_range": "{year}-9"}  # 填入d
    )
    report.add_fill_rules(
        excel_cell={"sheet": "附表 C.3 化石燃料燃烧排放表", "cell": "Q3", },
        content={"expression": "累积值(燃料消耗量)/3600", "date_range": "{year}-10"}  # 填入d
    )
    report.add_fill_rules(
        excel_cell={"sheet": "附表 C.3 化石燃料燃烧排放表", "cell": "R3", },
        content={"expression": "累积值(燃料消耗量)/3600", "date_range": "{year}-11"}  # 填入d
    )
    report.add_fill_rules(
        excel_cell={"sheet": "附表 C.3 化石燃料燃烧排放表", "cell": "S3", },
        content={"expression": "累积值(燃料消耗量)/3600", "date_range": "{year}-12"}  # 填入d
    )


def generate_report(year):
    global report
    out_file = os.path.join(os.path.dirname(__file__), "碳排放量计算1.xlsx")
    report.generate_report_excel(out_file=out_file, year=year)
    return out_file


if __name__ == "__main__":
    ...



"""
股票功能的入口文件
"""
import os.path
import sys
import traceback
from datetime import datetime

from pyecharts.commons.utils import JsCode
from pyecharts.globals import ThemeType
from pyecharts.options import CandleStickItem

from yangke.common.QtImporter import QtCore, Qt

import yangke.stock.globalVar as gv  # 所有的全局变量定义在该模块中，使用全局变量需要引入该模块
from yangke.common.config import logger
from yangke.stock.Project import Project
from yangke.stock.StockListView import StockItemDelegate
from yangke.common.crawler import start_scrapy_start

from yangke.common.qt import YkWindow, run_app, YkListViewWidget, YkInputPanel, YkDialog, YkItem, YkStandardItemModel, \
    html_widget, opts
from yangke.common.QtImporter import QComboBox, QFileDialog
from pyecharts.charts import Candlestick, Bar


class MainWindow(YkWindow):
    def __init__(self):
        super().__init__()  # 170, 124, 1244, 786
        if len(self.proj) > 0:
            self.project = Project(self.proj)
        else:
            self.project = Project()
        # self.enable_input_panel()
        self.symbols = self.get_stock_symbols()

        # ---------------------------- 初始化股票列表页 ------------------------------
        infos = self.get_stock_basic_infos()
        list_model = []
        for idx, row in infos.T.items():
            _ = {"name": row["symbol"], "名称": row["name"]}
            list_model.append(_)
        self.symbols_model = YkStandardItemModel(list_model)
        self.stock_list_lv = YkListViewWidget(self.symbols_model)
        self.stock_list_lv.setItemDelegate(StockItemDelegate())
        # noinspection all
        self.stock_list_lv.add_item_clicked_listener(self.stock_lv_item_clicked)
        self.add_panel("股票列表", self.stock_list_lv)
        # ---------------------------- 初始化股票列表页 ------------------------------

        self.start_ws_server()
        self.temp_ = None  # 用于传递临时数据的参数
        # self.add_content_tab(widget=html_widget(
        #     local_file=r"D:\PycharmProjects\lib4python\yangke\stock\ui\echarts\index.html"),
        #     tab_name="chart", replace=True)

    def stock_lv_item_clicked(self, index: QtCore.QModelIndex):
        """

        """

        # ------------------------- 获取股票的日线数据 ------------------------------------
        if isinstance(index, str):
            code = index
            name = self.project.tsd.get_name(code)
        else:
            data = index.data(Qt.UserRole)
            code = data.get("name")
            name = data.get("名称")
        df = self.project.tsd.download(code=code)
        df.sort_values(by='trade_date', inplace=True)
        # ------------------------- 获取股票的日线数据 ------------------------------------
        # ------------------------- 绘制K线图 ------------------------------------
        # x_data = df["trade_date"].apply(lambda x: str(x)[:10]).tolist()
        # y_data = df[["open", "close", "low", "high"]].values.tolist()
        df["sign"] = df.apply(lambda r: 1 if r['close'] >= r['open'] else -1, axis=1)  # 是否涨
        self.start_ws_server()
        _send_data = df[["trade_date", "open", "high", "low", "close", "vol", "sign"]].copy()
        _send_data['trade_date'] = _send_data['trade_date'].apply(lambda e: str(e)[:10])
        self.echarts_update_data({"data": _send_data})

        # ------------------------- 绘制K线图 ------------------------------------

    def get_stock_symbols(self):
        """
        获取A股所有的股票列表
        """
        symbols = self.project.tsd.get_all_stock_basic_info()
        return list(symbols['symbol'])

    def get_stock_basic_infos(self):
        info = self.project.tsd.get_all_stock_basic_info()
        return info

    def refresh(self):
        """
        刷新股票列表
        """
        infos = self.get_stock_basic_infos()
        list_model = []
        for idx, row in infos.T.items():
            _ = {"name": row["symbol"], "名称": row["name"]}
            list_model.append(_)
        self.symbols_model.update_item(list_model)

    def set_storage(self):
        """
        设置数据存储的位置
        """
        _ = os.path.abspath(os.path.join(os.path.dirname(__file__), "ui", "ui_panel.yaml"))
        input_panel = YkInputPanel(from_file=_, domain="set_storage", parent=self)
        dialog = YkDialog(self, widget=input_panel)
        dialog.set_size(400, 500)
        input_panel.apply_btn_connect()
        self.temp_ = {"input_panel": input_panel, "dialog": dialog}

        if self.project.storage is not None:
            box: QComboBox = input_panel.get_item("保存类型").value
            box.setCurrentIndex(0)
            box.setCurrentText(self.project.storage)

        # engine = create_engine(r'sqlite:///C:\path\to\foo.db')
        # self.storage = engine

    def _change_storage_type(self):
        input_panel: YkInputPanel = self.temp_.get("input_panel")
        type_ = input_panel.get_item("保存类型").get_value()
        input_panel.remove_item(index=list(range(1, input_panel.get_items_count())))
        if type_ == "本地目录":
            input_panel.append_item(
                YkItem("保存至本地目录", "", '<button on-click="choose_data_folder">选择文件</button>',
                       size=[50, 80, 40]),
            )
            input_panel.set_value("保存至本地目录", value=self.project.data_folder or "")
        elif type_ == "Sqlite数据库":
            input_panel.append_item(
                [YkItem("数据库名", "", "",
                        size=[50, 80, 40]),
                 ]
            )
            input_panel.set_value("数据库名", value=self.project.db_name or "")
        elif type_ == "Mysql数据库":
            input_panel.append_item(
                [
                    YkItem("服务器地址", "", "", size=[50, 80, 40]),
                    YkItem("端口号", "", "", size=[50, 80, 40]),
                    YkItem("用户名", "", "", size=[50, 80, 40]),
                    YkItem("密码", "", "", size=[50, 80, 40]),
                    YkItem("数据库名", "", "", size=[50, 80, 40]),
                ]
            )
            input_panel.set_value("服务器地址", value=self.project.db_ip or "")
            input_panel.set_value("端口号", value=self.project.db_port or "")
            input_panel.set_value("用户名", value=self.project.db_user or "")
            input_panel.set_value("密码", value=self.project.db_passwd or "")
            input_panel.set_value("数据库名", value=self.project.db_name or "")

    def save_storage(self):
        input_panel: YkInputPanel = self.temp_.get("input_panel")
        info = input_panel.get_values_and_units(need_unit=False, need_label=True, need_dict=True)
        self.project.set_storage(info.get("保存类型"))
        if self.project.storage == "Sqlite数据库":
            self.project.db_name = info.get("数据库名")
        elif self.project.storage == "本地目录":
            self.project.data_folder = info.get("保存至本地目录")
        elif self.project.storage == "Mysql数据库":
            self.project.db_user = info.get("用户名")
            self.project.db_passwd = info.get("密码")
            self.project.db_ip = info.get("服务器地址")
            self.project.db_port = info.get("端口号")
            self.project.db_name = info.get("数据库名")
        dialog: YkDialog = self.temp_.get("dialog")
        dialog.close()

    def choose_data_folder(self):
        """
        选择股票数据文件保存的路径
        """
        data_folder = QFileDialog.getExistingDirectory(self, '选择错误工况模型保存目录', os.getcwd(),
                                                       QFileDialog.ShowDirsOnly | QFileDialog.DontResolveSymlinks)
        if data_folder:
            self.project.data_folder = data_folder
            self.temp_.get("input_panel").set_value("保存至本地目录", data_folder)

    def save(self):
        self.proj = self.project.to_dict()
        super().save()

    def save_as(self):
        self.proj = self.project.to_dict()
        super().save_as()


def start():
    start_time = datetime.datetime.now()  # 用来统计执行时间

    if gv.command == "download":
        gv.tsd.download(gv.symbol, append=True)

    elif gv.command == "getName":
        gv.tsd.get_name()

    elif gv.command == "getOpenLimitUp":
        gv.tsd.get_open_limit('up')

    elif gv.command == "getOpenLimitDown":
        gv.tsd.get_open_limit_and_lastdays('down')

    elif gv.command == "getSymbols":
        symbols = gv.tsd.get_all_stock_basic_info()  # 会生成allsymbols.csv
        # for symbol in symbols:#传递参数太多，不适合outputStream传递，使用硬盘文件传递
        #     print("py to java:{}".format(symbol))
        # print(np.array(symbols))

    elif gv.command == "getStockNum":
        symbols = gv.tsd.get_all_stock_basic_info()
        print("共有 {} 只股票".format(len(symbols)))

    elif gv.command == "getNewStocks":
        # 查询当前所有正常上市交易的股票列表 ts_code     symbol     name     area industry    list_date(上市日期)
        data = gv.tsd.api.query('stock_basic', exchange='', list_status='L',
                                fields='ts_code,symbol,name,area,industry,list_date')
        sortedData = data.sort_values(by="list_date", ascending=False)
        for index, row in sortedData[0:20].iterrows():
            print(row["list_date"], row["name"], row["area"], row["industry"], row["symbol"])

    elif gv.command == "getBottomOut":
        symbols, _ = gv.tsd.get_bottom_out(int(gv.symbol))
        print("{}天内触底反弹股票列表如下：".format(gv.symbol))
        print(symbols)

    elif gv.command == "debug":
        gv.tsd.download(gv.symbol, append=True)
        # tsd.download_daily(date.today())
        # gv.tsd.get_open_limit()
        # gv.tsd.get_bottom_out(10)
        # logging.debug("最后一个交易日：{}".format(tsd.get_last_working_day()))
        # logging.debug("查询股票{}今天的交易数据：".format(symbol, tsd.get_data_of_day(symbol)))
        # gv.logger.debug("目前上市的股票代码：\n{}".format(pd.Series(gv.tsd.get_all_stock_symbol())))
        # gv.logger.debug("最近的工作日列表:\n{}".format(pd.Series(gv.tsd.get_last_n_working_day(30))))
        # tsd.get_open_limit_of_n_trading_days(30, 'up', 'ascend')
        # gv.tsd.get_open_limit_of_n_trading_days(30, 'down', 'ascend')
        # gv.tsd.get_open_limit_and_lastdays(last_date=datetime.date.fromisoformat("2019-12-31"))
        # gv.tsd.recent_30_open_limit_down_to_file(days=30)

        # 模拟交易
        symbols = gv.tsd.get_all_stock_basic_info()
        # db.test1()
        # Main.test1()

        # 神经网络预测
        # pad.test1()
        # pad.test2()
        # pad.test3()
        # pad.test4()
        # datafile = os.path.join(gv.dataDirectory, gv.symbol + '.csv')
        # pps.prediction1(datafile)

        # import prediction.pytorch_pred as ptp
        # ptp.start_mysql_service()

        import yangke.stock.prediction.pytorch_pred_stock as ptps

        ptps.lstm_stock_use_in_build_rnn_fit(gv.symbol)
        # ptps.lstm_stock_classify(gv.symbol)

        # start_mysql_service()

        start_scrapy_start(r"start_stock10jqka.py")
        # start_scrapy_spider(r"jqka_spider.py")  # 因为在start_stock10jqka.py中还初始化了数据库，因此只能从start_...开始执行爬虫

        # stocks = gv.tsd.download_all_stocks(exclude=['000029', '600145', '688086', '601696'])  # 排除掉两个停牌了很多年的傻逼股票
        # pred_df = ptps.prediction(stocks, 'fit')
        # pred_df.sort_values(by=['p_change'], inplace=True, ascending=False)
        # gv.logger.info("预测涨幅最大的5只股票为：\n{}".format(pred_df[:5]))

    else:
        gv.logger.error("Unknown command: {}".format(gv.command))
        sys.exit(0)
    end_time = datetime.datetime.now()
    print("执行用时：{}s".format((end_time - start_time)))


if __name__ == "__main__":
    # run_app(MainWindow, theme='dark_lightgreen.xml')
    run_app(MainWindow, theme='default_dark.xml')
    # run_app(MainWindow, None)



# sendInput方法发送键盘鼠标事件
import ctypes
import time
import pydirectinput

from yangke.common.win.keyboard import get_virtual_keycode

SendInput = ctypes.windll.user32.SendInput

PUL = ctypes.POINTER(ctypes.c_ulong)


def key_click_dx(x, y):
    pydirectinput.click(x, y)


def key_double_click_dx(x, y):
    pydirectinput.doubleClick()


def post_key_dx(key):
    """
    key = "esc", "A", "shift"等
    """
    pydirectinput.press(key)


class KeyBdInput(ctypes.Structure):
    _fields_ = [
        ("wVk", ctypes.c_ushort),
        ("wScan", ctypes.c_ushort),
        ("dwFlags", ctypes.c_ulong),
        ("time", ctypes.c_ulong),
        ("dwExtraInfo", PUL)
    ]


class HardwareInput(ctypes.Structure):
    _fields_ = [
        ("uMsg", ctypes.c_ulong),
        ("wParamL", ctypes.c_short),
        ("wParamH", ctypes.c_ushort)
    ]


class MouseInput(ctypes.Structure):
    _fields_ = [
        ("dx", ctypes.c_long),
        ("dy", ctypes.c_long),
        ("mouseData", ctypes.c_ulong),
        ("dwFlags", ctypes.c_ulong),
        ("time", ctypes.c_ulong),
        ("dwExtraInfo", PUL)
    ]


class Input_I(ctypes.Union):
    _fields_ = [
        ("ki", KeyBdInput),
        ("mi", MouseInput),
        ("hi", HardwareInput),
    ]


class Input(ctypes.Structure):
    _fields_ = [
        ("type", ctypes.c_ulong),
        ("ii", Input_I)
    ]


class POINT(ctypes.Structure):
    _fields_ = [
        ("x", ctypes.c_ulong),
        ("y", ctypes.c_ulong)
    ]


def press_key(key):
    hex_key_code = get_virtual_keycode(key)
    extra = ctypes.c_ulong(0)
    ii_ = Input_I()
    ii_.ki = KeyBdInput(0, hex_key_code, 0x0008, 0, ctypes.pointer(extra))
    x = Input(ctypes.c_ulong(1), ii_)
    ctypes.windll.user32.SendInput(1, ctypes.pointer(x), ctypes.sizeof(x))


def release_key(key):
    hex_key_code = get_virtual_keycode(key)
    extra = ctypes.c_ulong(0)
    ii_ = Input_I()
    ii_.ki = KeyBdInput(0, hex_key_code, 0x0008 | 0x0002, 0, ctypes.pointer(extra))
    x = Input(ctypes.c_ulong(1), ii_)
    ctypes.windll.user32.SendInput(1, ctypes.pointer(x), ctypes.sizeof(x))


def post_key(key, last_time=0.1):
    """
    前端按键，无法发送组合键
    """
    press_key(key)
    time.sleep(last_time)
    release_key(key)


def get_mouse_pos():
    """
    获取鼠标位置
    """
    orig = POINT()
    ctypes.windll.user32.GetCursorPos(ctypes.byref(orig))
    return int(orig.x), int(orig.y)


def set_mouse_pos(x, y):
    ctypes.windll.user32.SetCursorPos(x, y)


def mouse_press(x, y, button='left'):
    i = 2
    if button == 'right':
        i = 8
    set_mouse_pos(x, y)
    extra = ctypes.c_ulong(0)
    ii_ = Input_I()
    ii_.mi = MouseInput(0, 0, 0, i, 0, ctypes.pointer(extra))
    x = Input(ctypes.c_ulong(0), ii_)
    ctypes.windll.user32.SendInput(1, ctypes.pointer(x), ctypes.sizeof(x))


def mouse_release(x, y, button='left'):
    i = 4
    if button == 'right':
        i = 8
    set_mouse_pos(x, y)
    extra = ctypes.c_ulong(0)
    ii_ = Input_I()
    ii_.mi = MouseInput(0, 0, 0, i, 0, ctypes.pointer(extra))
    x = Input(ctypes.c_ulong(0), ii_)
    ctypes.windll.user32.SendInput(1, ctypes.pointer(x), ctypes.sizeof(x))



from yangke.stock.dataset.tushareData import StockData


class Project:
    def __init__(self, settings=None, encoding='utf8', data_folder=None, cmd=None,
                 symbol=None, storage=None):
        if settings is not None and len(settings) > 0:
            self.encoding = settings.get("encoding")
            self.cmd = settings.get("cmd")
            self.symbol = settings.get("symbol")
            self.storage = settings.get("storage")  # 记录存储类型，可取值"Mysql数据库", "Sqlite数据库"或"本地目录"
            self.data_folder = settings.get("data_folder")  # 本地目录存储时需要
            self.db_name = settings.get("db_name")  # sqlite或mysql数据库存储时需要
            self.db_user = settings.get("db_user")  # mysql数据库存储时需要
            self.db_passwd = settings.get("db_passwd")  # mysql数据库存储时需要
            self.db_ip = settings.get("db_ip")  # mysql数据库存储时需要
            self.db_port = settings.get("db_port")  # mysql数据库存储时需要
        else:
            self.encoding = encoding
            self.data_folder = data_folder  # 股票数据存储的目录
            self.cmd = cmd  # 调用程序的命令
            self.symbol = symbol  # 操作哪只股票，symbol对应股票代码
            self.storage = storage or 'Sqlite数据库'  # 数据的存储方式，默认是mysql，也可以取值为"file"，表示本地存储
            self.data_folder = None  # 本地目录存储时需要
            self.db_name = None  # sqlite或mysql数据库存储时需要
            self.db_user = None  # mysql数据库存储时需要
            self.db_passwd = None  # mysql数据库存储时需要
            self.db_ip = None  # mysql数据库存储时需要
            self.db_port = None  # mysql数据库存储时需要
        self.tsd = StockData(self)  # StockData类的对象，其构造函数会初始化tushare账户的口令

    def to_dict(self):
        return {
            "encoding": self.encoding,
            "data_folder": self.data_folder,
            "tsd": self.tsd,
            "cmd": self.cmd,
            "symbol": self.symbol,
            "storage": self.storage,
            "db_name": self.db_name,
            "db_user": self.db_user,
            "db_passwd": self.db_passwd,
            "db_ip": self.db_ip,
            "db_port": self.db_port,
        }

    def set_storage(self, storage):
        self.tsd.storage = storage
        self.storage = storage

    def get_storage(self):
        return self.storage



# 该模块必须位于common文件夹下
import sys
import traceback
import inspect
from typing import Optional

from yangke.base import get_settings, run_once
import os
# ------------------------------ 使用loguru时的代码 --------------------------------------
from loguru import logger as logger1
import loguru

# noinspection All
Logger = loguru._logger.Logger
# 该类中一律使用logger，不使用全局变量模块中的gv.logger，这样就无须引入全局变量模块.
# 调用模块会将gv.logger设置为这里的logger，但因为该类可以直接访问自己的logger，就无须且不建议使用gv.logger
logger: Logger = logger1
# ------------------------------ 使用loguru时的代码 --------------------------------------

# ------------------------------ 使用nb_log时的代码 --------------------------------------
# from logging import Logger
# from nb_log import get_logger
# logger = get_logger(None)
# ------------------------------ 使用nb_log时的代码 --------------------------------------
# noinspection all
level: int = None
# 这里声明以下变量，这样在其他模块中引用config时无须引入logging模块即可使用以下变量
CRITICAL = 50
FATAL = CRITICAL
ERROR = 40
WARNING = 30
WARN = WARNING
INFO = 20
DEBUG = 10
NOTSET = 0
log_file_id = None

"""
Level name      Severity value      Logger method
TRACE           5                   logger.trace
DEBUG           10                  logger.debug
INFO            20                  logger.info
SUCCESS         25                  logger.success
WARNING         30                  logger.warning
ERROR           40                  logger.error
CRITICAL        50                  logger.critical
"""


@run_once
def initLogger(level_color=None, format_color=None, log_format=None,
               date_format=None, diagnose=True) -> Logger:
    """
    levelColor和formatColor的取值示例如下，具体配置说明参考https://loguru.readthedocs.io/en/stable/api/logger.html#color

        level_color = {
            "DEBUG": 'green',
            "INFO": 'blue',
            "WARN": 'red',
            "ERROR": 'red d',
            "FATAL": 'red'}
        format_color = {
            "time": "green",
            "level": "light
            }

    :param date_format: 日志中日期的格式
    :param log_file: 日志保存到的文件，文件名中可以引用time，如"runtime_{time}.log"，则time会被自动替换为实时时间
    :param log_format: 日志格式
    :param diagnose: 运行出错时，是否追踪错误原因
    :param level: 日志级别，可以为logging.DEBUG等等，具体参见logging类的日志级别
    :param level_color: 各日志级别的字体配置，为字典类型，可取值"default"使用默认配色
    :param format_color: 日志各个格式的输出字体配置，可取值"default"使用默认配色
    :return: logger，使用该logger进行日志输出。
    """

    # def sink(msg):
    #     print(msg)
    #     record = msg.record
    #     file = record["file"].name
    #     function = record["function"]
    #     line = record["line"]
    #     message = record['message']
    #     module = record["module"]
    #     name = record["name"]
    #
    #     if record.level.no == DEBUG:
    #         pass

    def setLevel(logger_cls, *args, **kwargs):
        logger.debug("暂不支持中途更改日志级别")

    Logger.setLevel = setLevel

    if date_format is None:
        date_format = "YYYY-MM-DD HH:mm:ss"
    if log_format is None:
        log_format = "{time:" + date_format + "} - {level} - {module}:{function}:{line} - {message}"
    else:
        if "{time:" not in log_format:  # 日志格式中没有添加日期，则添加
            log_format = log_format.replace("{time", "{time:" + date_format)

    def assemble_log_fmt(color):
        if " " not in color:
            log_fmt = f"<{color}>" + log_format + "</>\n"
        else:  # 说明color中包含color和style两个设置项，都解析出来，顺序无所谓
            color = color.split(" ")[0]
            style = color.split(" ")[1]
            log_fmt = f"<{color}><{style}>" + log_format + "</></>\n"
        return log_fmt

    def format_(record):
        """
        当用户自定义日志格式时，调用该方法生成格式字符串

        :param record:
        :return:
        """
        color = "white"
        if record['level'].no == DEBUG:
            if level_color is not None:
                color = level_color.get('DEBUG') or 'green'
            else:
                color = 'green'
        elif record['level'].no == INFO:
            if level_color is not None:
                color = level_color.get('INFO') or 'blue'
            else:
                color = 'blue'
        elif record['level'].no == WARN:
            if level_color is not None:
                color = level_color.get('WARN') or 'red'
            else:
                color = 'red'
        elif record['level'].no == ERROR:
            if level_color is not None:
                color = level_color.get('ERROR') or 'RED'
            else:
                color = 'RED'
        elif record['level'].no == CRITICAL:
            if level_color is not None:
                color = level_color.get('CRITICAL') or 'blue RED'
            else:
                color = 'blue RED'
        return assemble_log_fmt(color)

    global logger
    if level_color is not None or format_color is not None:
        logger.remove(0)
        level = get_settings().get_settings("logger.level") or 10
        logger.add(sys.stdout, level=level, diagnose=diagnose, format=format_, colorize=True)

    return logger


def holdLoggingLevel(para='end', outer=True):
    """
    临时提高日志级别，例如更改前使用holdLoggingLevel(logging.WARN)将日志级别临时更改为“WARN”，
    更改后调用holdLoggingLevel('end')恢复日志级别为临时更改前的级别。
    holdLoggingLevel(level)和holdLoggingLevel('end')构成一个区间。
    多个区间可以嵌套，以最外层区间的设置为最终设置。如果需要以内部区间的设置为最终设置，将outer设为False即可。

    outer=False和outer=True优先级相同，后设置的生效。

    使用示例：

    ...any code...                                             \n
    holdLoggingLevel(logging.DEBUG, False)  # 1                \n
    ...DEBUG生效...                                            \n
    holdLoggingLevel('logging.WARN, False)  # 2                \n
    ...WARN生效...                                              \n
    holdLoggingLevel('logging.DEBUG, False)  # 3                \n
    ...DEBUG生效...                                             \n
    holdLoggingLevel('logging.WARN')  # 4                       \n
    ...WARN生效...                                              \n
    holdLoggingLevel('logging.DEBUG')  # 5                      \n
    ...WARN生效...，因为outer=True，以当前设置更外层的为准       \n
    holdLoggingLevel('logging.ERROR')  # 6                      \n
    ...WARN生效...，因为outer=True，一直找到最外层的日志级别      \n
    holdLoggingLevel('end')  # 关闭6                            \n
    ...WARN生效...                                              \n
    holdLoggingLevel('end')  # 关闭5                            \n
    ...WARN生效...                                              \n
    holdLoggingLevel('end')  # 关闭4                            \n
    ...DEBUG生效...，因为4对应的设置已经关闭，则以3为准           \n
    holdLoggingLevel('end',outer=False)                        \n
    ...WARN生效...，因为3对应的设置已经关闭，则以2为准            \n
    holdLoggingLevel('end',outer=False)                        \n
    ...DEBUG生效...                                            \n
    ...DEBUG生效...如果outer=False，且与之对应的end语句一直不出现，则级别设置一直到程序结束都生效
    """
    # global level
    # global prior  # 区间嵌套层数
    # if outer:  # 如果外部优先
    #     if para in [DEBUG, INFO, WARN, ERROR, CRITICAL]:
    #         prior += 1
    #         if prior == 1:  # 如果是最外层就操作
    #             level = logger.getEffectiveLevel()  # 记录临时修改前日志级别
    #             logger.setLevel(para)
    #     elif type(para) == str and para.lower() == 'end':
    #         # 如果第一个变量为False，则不会判断第二个变量，所以即使para不是字符串，para.lower()不会报错
    #         prior -= 1
    #         if prior == 0:
    #             logger.setLevel(level)
    #     else:
    #         logger.error('参数未识别，para只能取值logging日志的有效级别或字符串"end"!')
    # else:  # 如果内部优先
    #     if para in [logging.DEBUG, logging.INFO, logging.WARN, logging.ERROR, logging.CRITICAL]:
    #         level = logger.getEffectiveLevel()
    #         logger.setLevel(para)
    #     elif type(para) == str and para.lower() == 'end':
    #         # 恢复日志级别为外部级别
    #         logger.setLevel(level)
    #     else:
    #         logger.error('参数未识别，para只能取值logging日志的有效级别或字符串"end"!')
    pass


def smart_decorator(decorator):
    """
    智能装饰器，用于装饰装饰器，使得装饰器无论在有没有参数的情况下可以使用同样的语法。无论是类装饰器和方法装饰其都可以装饰。

    :param decorator:
    :return:

    当func is None时，
    decorator_proxy=smart_decorator(decorator)

    decorator_proxy1=decorator_proxy(func, **kwargs)
    """

    def decorator_proxy(func_method=None, *args, **kwargs):
        if func_method is not None:
            return decorator(func_method, *args, **kwargs)

        def decorator_proxy1(func_method1):
            return decorator(func_method1, *args, **kwargs)

        return decorator_proxy1

    return decorator_proxy


@smart_decorator
def loggingTitleCall(func, title=None):
    """
    装饰器 @loggingTitleCall(title=title)，用于在方法前后打印日志标题和结尾

    疑问：

        这里func是个必要参数，为什么使用时可以不指定func的值？

    解答：

        在带参数使用时， @loggingTitleCall(title='title')相当于调用 loggingTitleCall(title='title')(func=FunctionType)，func默认赋值了被装饰的方法或类名。

        不带参数使用时， @loggingTitleCall 相当于调用 loggingTitleCall(func=FunctionType, title="新环境")。如果该装饰器参数列表中不指定title的默认值，则使用时必须给title赋值。

        不能使用 @loggintTitleCall('title')，即必须将参数写作title="title"，否则，解释器会把'title'赋值给func造成逻辑错误。

    :param func:
    :param title:
    :return:
    """
    if title is None:
        title = func.__name__

    def wrap_f(*args, **kwargs):
        loggingTitle(title=title, start_end='start')
        try:
            result = func(*args, **kwargs)
        except TypeError as e:
            if e.args[0] == "'str' object is not callable":
                logger.error("貌似使用了带参数的装饰器，如需要传递参数，请指定title='参数'")
            traceback.print_exc()
            exit(1)
        loggingTitle(title=title, start_end="end")
        return result

    return wrap_f


@smart_decorator
def SqliteClass(cls, table=None):
    """
    一个装饰器，在其他类上标记@SqliteClass，使得其他类可以用于和sqlite互转

    :param cls:
    :param table:
    :return:
    """
    cls.__tablename__ = 'user'
    columns = [*cls.__init__.__code__.co_names]
    sig = inspect.signature(cls.__init__)
    for par in sig.parameters.values():
        if par.annotation is not inspect._empty:
            print('Parameter: ', par)
            print("name: ", par.name)
            print("type: ", par.annotation)

    class NewClass():
        def __init__(self, *args, **kwargs):
            self.new = cls(*args, **kwargs)

    return NewClass


def loggingTitle(title: str = "default", logger=None, start_end: str = "start"):
    """
    输出以下格式的日志标题
    +-----------------------------------------+
    |                   title                 |
    +-----------------------------------------+
    :param start_end: 可以取值"start"和"end"
    :param title:
    :return:
    """
    if logger is None:  # 没有传入logger，则使用全局logger
        logger = globals().get('logger')

    n, m = __cal_char_length(title)

    if start_end.lower() == "start":
        start_char, end_char = "-", "↓"
        logger.debug("'{}'环境准备中...\n".format(title))
        logger.debug("+" + start_char * n + "+")
        logger.debug("|" + " " * m + title + " " * m + "|")
        logger.debug("+" + end_char * n + "+")
    elif start_end.lower() == "end":
        start_char, end_char = "↑", "-"
        logger.debug("+" + start_char * n + "+")
        logger.debug("|" + " " * m + title + " " * m + "|")
        logger.debug("+" + end_char * n + "+\n")


def __cal_char_length(title: str) -> (int, int):
    """
    获得日志标题边框行和标题行中填充字符的数量

    :return: number_of_border_line, number_of_title_line
    """
    length, _, _ = get_char_number(title)
    if length < 100:
        if length % 2 == 0:  # 标题内容长度是偶数
            number_of_border_line = 100
            number_of_title_line = int(50 - length / 2)
        else:  # 标题内容长度是奇数
            number_of_border_line = 101
            number_of_title_line = int((101 - length) / 2)
    else:
        number_of_border_line = length + 2
        number_of_title_line = 1
    return number_of_border_line, number_of_title_line


def get_char_number(content: str) -> (int, int, int):
    """
    获得字符串的占位长度、汉字个数、字母个数。
    占位长度即输出式占的宽度，汉字占两个长度，英文数字等占一个长度；
    默认的len(string)函数获得是字数，汉字和字符都算一个长度
    :param content: 字符串内容
    :return: length - 汉字算两个长度，字母算一个长度；
    number_Of_Chinese_Chars - 汉字个数；
    number_of_letters - 其他字母个数
    """
    lenText = len(content)  # 总字数，汉字和其他字符都算一个长度
    lenChar = len(content.encode("utf8"))  # 这种长度汉字算3个，其他字符算1个
    number_Of_Chinese_Chars = int((lenChar - lenText) / 2)
    number_of_letters = lenText - number_Of_Chinese_Chars
    length = number_Of_Chinese_Chars * 2 + number_of_letters
    return length, number_Of_Chinese_Chars, number_of_letters


def printInColor(text: str = "",
                 color_fg: str = 'blue', color_bg: str = 'yellow', mode: int = 0,
                 localSet='inline', end=os.linesep):
    """
    以特定颜色输出文本内容

    color_fg和color_bg可以取以下值：

    "white", "red", "green", "yellow", "blue", "purple", "cyan", "gray"

    mode可取以下值：

    0        |     终端默认设置
    1        |     高亮显示
    22       |     非粗体
    4        |     下划线
    24       |     非下划线
    5        |     闪烁
    25       |     非闪烁
    7        |     反显
    27       |     非反显

    :param end:
    :param text: 输出的文本内容，如果只是设置终端输出格式，可以不赋值
    :param color_fg: 终端输出文字的前景色
    :param color_bg: 终端输出的文字背景色
    :param mode: 显示模式
    :param localSet: 'start'、'end'、'inline'，格式开始生效、格式结束生效、格式仅在当前内容生效
    """
    colors_names_list = ["white", "red", "green", "yellow", "blue", "purple", "cyan", "gray", ""]
    colors_fg = [30, 31, 32, 33, 34, 35, 36, 37, '']
    colors_bg = [40, 41, 42, 43, 44, 45, 46, 47, '']

    mode: str = str(mode) + ";"
    code_fg = str(colors_fg[colors_names_list.index(color_fg)]) + ";"
    code_bg = str(colors_bg[colors_names_list.index(color_bg)]) + ";"
    format_str = (mode + code_fg + code_bg).rstrip(';')  # 删掉最后一个分号
    if localSet.lower() == 'start':
        print("\033[{}m".format(format_str), end=end)
    elif localSet.lower() == 'end':
        print("\033[0m", end=end)
    else:
        print("\033[{}m{}\033[0m".format(format_str, text), end=end)


def remove_file_logger():
    """
    删除logger对象对文件的输出

    :param file_id:
    :return:
    """
    if log_file_id is not None:
        logger.remove(log_file_id)


def add_file_logger(log_file: Optional[str] = "default", rotation="50 MB", retention="1 days", diagnose=True):
    """

    :param diagnose: 运行出错时，是否追踪错误原因
    :param retention: 文件日志输出时，指定保留日志的期限，如“30 days”表示只保留最近30天的日志
    :param rotation: 文件日志输出时，输出下一个日志的条件，可以是大小，也可以是时间，如"50 MB"、"00:00", "1 week"等，具体参见loguru
    :param log_file:
    :return:
    """
    global logger, log_file_id
    settings = get_settings()
    if settings.get_settings("logger.logFile") == "None":
        log_file = None
    level = settings.get_settings("logger.level") or 10
    if log_file == "default":
        log_file = "runtime_{time}.log"

    if log_file is not None:
        log_file_id = logger.add(log_file, encoding="utf8", level=level, rotation=rotation, retention=retention,
                                 diagnose=diagnose)


def __import_first__():
    global logger
    # 当被其他模块首次引用时，会执行以下初始化代码
    settings = get_settings().get('logger') or {}
    levelColor = settings.get('levelColor')  # 可以为None
    formatColor = settings.get('formatColor')  # 可以为None
    logFormat = settings.get('format')
    dateFormat = settings.get('dateFormat')
    logger = initLogger(level_color=levelColor, format_color=formatColor, log_format=logFormat,
                        date_format=dateFormat)


prior = 0  # 用来控制临时修改日志级别的变量
__import_first__()



import sys
import traceback

from flask import Flask
from jinja2 import Environment, FileSystemLoader
from markupsafe import Markup
from pyecharts.globals import CurrentConfig

# 关于 CurrentConfig，可参考 [基本使用-全局变量]
CurrentConfig.GLOBAL_ENV = Environment(loader=FileSystemLoader("./templates"))

from pyecharts import options as opts
from pyecharts.charts import Bar

app = Flask(__name__, static_folder="templates")


def bar_base() -> Bar:
    c = (
        Bar()
        .add_xaxis(["衬衫", "羊毛衫", "雪纺衫", "裤子", "高跟鞋", "袜子"])
        .add_yaxis("商家A", [5, 20, 36, 10, 75, 90])
        .add_yaxis("商家B", [15, 25, 16, 55, 48, 8])
        .set_global_opts(title_opts=opts.TitleOpts(title="Bar-基本示例", subtitle="我是副标题"))
    )
    return c


@app.route("/")
def index():
    c = bar_base()
    return Markup(c.render_embed())


if __name__ == "__main__":
    try:
        if len(sys.argv) >= 2:
            port = int(sys.argv[1])  # ['server.py', port]
        else:
            port = 5000
        app.run(port=port)
    except:
        traceback.print_exc()



"""
定义stocks软件中需要存储到数据库的类
"""
from sqlalchemy import Column, Integer, String, Date, CHAR, Table, MetaData, DateTime, VARCHAR

holiday_table = MetaData()  # 没有关系的表放在不同的metadata中
symbols_table = MetaData()

Table('holiday', holiday_table,
      Column('calendarDate', Date, primary_key=True),
      Column('isOpen', CHAR, nullable=False))

Table("symbols", symbols_table,
      Column('symbol', Integer, primary_key=True),
      Column('name', String(20), nullable=False)
      )



import copy
import math
from typing import List

import pandas as pd
import os

from yangke.base import plot_2d_scatter, yield_all_file, interpolate_value_complex, plot_2d_line, \
    update_matplotlib_font_config, get_datetime_col_of_dataframe
from yangke.core import str_in_list
from yangke.common.fileOperate import read_csv_ex, write_excel
from yangke.common.config import logger

velocity_min = 3  # 切入风速m/s
velocity_max = 20  # 切出风速m/s
diameter = 141  # 风力发电机转轮直径，单位m
r0 = 287.05  # 干燥空气的气体常数，单位J/(kg.K)
rho0 = 1.225
p0 = 87580  # 87580  # 替代的大气压力，如果没有大气压力测点，但计算过程需要大气压力，则会取该值
# 数据文件中应包括风速、有功功率、温度、大气压力
data_file = r"E:\热工院\2021\101工程风光储\导数\Data_20210702032750\A环线\SNTY02_20210101_20210702_ten_avg_val.csv"
data_folder = r"E:\热工院\2021\101工程风光储\导数\Data_20210702032750"
velocity_wind_design = [3, 3.5, 4, 4.5, 5, 5.5, 6, 6.5, 7, 7.5, 8, 8.5, 9, 9.5, 10, 10.5, 11, 11.5, 12, 12.5, 13, 13.5,
                        14, 14.5, 15, 15.5, 16, 16.5, 17, 17.5, 18, 18.5, 19, 19.5, 20]
power_design = [45, 144, 254, 386, 544, 733, 958, 1223, 1532, 1886, 2288, 2730, 2989, 3116, 3176, 3198, 3200, 3200,
                3200, 3200, 3200, 3200, 3200, 3200, 3200, 3200, 3200, 3200, 3200, 3200, 3200, 3200, 3200, 3200, 3200]
power_design = [x * 1000 for x in power_design]
x_ticks = [x for x in range(0, 21, 4)]
y_ticks = [y for y in range(0, 4001, 500)]


class SectionData:
    """
    风力发电机的数据类，记录风力发电机的各类原始及计算数据
    """

    def __init__(self, section=None, dataframe: pd.DataFrame = None):
        self.section = section  # 风速区间
        self.dataframe = dataframe  # 在此风速区间内的所有运行数据点
        self.diameter = None  # 风力发电机转轮直径 m
        self.area = None  # 风机转轮扫略面积 m2
        self.velocity_mean = None  # 所有运行数据点的平均风速 m/s
        self.velocity_mean_standard = None  # 所有运行数据点的标准风速平均值
        self.power_mean = None  # 所有运行数据点的平均功率 W
        self.power_mean_standard = None  # 所有运行数据点的标准功率平均值
        self.temperature_mean = None  # 所有运行数据点的平均温度 K
        self.pressure_mean = None  # 所有运行数据点的平均气压 Pa
        self.rho_mean = None  # 所有运行数据点的平均空气密度 kg/m3
        self.ita = None  # 本风速区间对应的功率系数 %
        self.ita_standard = None  # 本风速区间对应的标准功率系数
        self.num_of_data = None  # 本风速区间的数据记录个数
        self.power_wind = None  # 本风速区间的平均风功率
        self.power_wind_standard = None  # 本风速区间的标准风功率的平均值
        self.rho_0 = None

    def set_wind_turbine_info(self, diameter):
        self.diameter = diameter
        self.area = math.pi * diameter * diameter / 4

    def calculate_wind_turbine(self):
        mean = self.dataframe.mean(numeric_only=True)
        self.velocity_mean = mean["风速"]
        self.power_mean = mean["有功功率"]
        self.temperature_mean = mean["温度"]
        if self.temperature_mean < 200:  # 如果环境温度单位是℃，则转换为K
            self.temperature_mean = self.temperature_mean + 273.15
        self.pressure_mean = mean["大气压力"]
        if self.pressure_mean < 70000:  # 如果大气压力的单位是KPa，则转换为Pa
            self.pressure_mean = self.pressure_mean * 1000
        self.rho_mean = self.pressure_mean / r0 / self.temperature_mean
        self.power_wind = 0.5 * self.area * self.rho_mean * math.pow(self.velocity_mean, 3)

        self.ita = 2 * self.power_mean / (self.rho_mean * self.area * math.pow(self.velocity_mean, 3)) * 100

        self.velocity_mean_standard = mean["标准风速"]
        self.power_mean_standard = mean["标准功率"]
        self.power_wind_standard = 0.5 * self.area * self.rho_mean * math.pow(self.velocity_mean_standard, 3)
        self.ita_standard = 2 * self.power_mean_standard / \
                            (rho0 * self.area * math.pow(self.velocity_mean_standard, 3)) * 100

    def set_data_num(self, num):
        self.num_of_data = num


def fix_data(file):
    """
    确保数据列完备，至少包括["风速", "有功功率", "温度", "大气压力"]


    :param file:
    :return:
    """
    data0: pd.DataFrame = read_csv_ex(file)
    columns = data0.columns
    columns_need = ["风速", "有功功率", "温度", "大气压力"]
    column_name = {}
    # 处理必要数据列：风速
    exist_v = str_in_list("风速", columns, need_item=True)  # 如果某一列的列名包含“风速”则摘出该列名，将列明更改为风速
    if exist_v[0]:
        column_name.update({exist_v[1]: "风速"})
    else:
        logger.error("数据中不存在必要数据列：风速")
    # 处理非必要数据列：温度
    exist_temp = str_in_list("温度", columns, need_item=True)
    if exist_temp[0]:
        column_name.update({exist_temp[1]: "温度"})
    else:
        exist_temp = str_in_list("气温", columns, need_item=True)
        if exist_temp[0]:
            column_name.update({exist_temp[1]: "温度"})
    # 处理非必要数据列：大气压力
    exist_pressure = str_in_list("大气压力", columns, need_item=True)
    if exist_pressure[0]:
        column_name.update({exist_pressure[1]: "大气压力"})
    else:
        exist_temp = str_in_list("压力", columns, need_item=True)
        if exist_temp[0]:
            column_name.update({exist_temp[1]: "大气压力"})
        else:
            logger.debug("未找到大气压力列")
            data0["大气压力"] = p0
    # 处理必要数据列：有功功率
    exist_power = str_in_list("有功功率", columns, need_item=True)
    if exist_power[0]:
        column_name.update({exist_power[1]: "有功功率"})
    else:
        exist_power = str_in_list("功率", columns, need_item=True)
        if exist_power[0]:
            column_name.update({exist_power[1]: "有功功率"})
        else:
            logger.error("数据中不存在必要数据列：有功功率")

    data0.rename(columns=column_name, inplace=True)
    col = get_datetime_col_of_dataframe(data0)
    if col is None:
        import datetime
        time = datetime.datetime(2021, 1, 1, 0, 0, 0)
        time_series = []
        for idx in range(data0.shape[0]):
            time_series.append(time)
            time = time + datetime.timedelta(minutes=10)
        data0["DateTime"] = time_series
    else:
        data0.rename(columns={col: "DateTime"}, inplace=True)
    return data0


def filter_data(data0):
    """
    筛选数据，获得在切入风速和切出风速之间正常运行的风机运行数据，同时统一数据单位为国际单位

    :param data0: 数据dataframe
    :return:
    """
    logger.debug(f"总数据量：{data0.shape}")
    data1 = data0.dropna()
    logger.debug(f"非空数据量：{data1.shape}")
    data2 = data1[data1["有功功率"] > 0]
    logger.debug(f"风力发电机在服务状态数据量：{data2.shape}")
    data3 = data2[data2["风速"] >= velocity_min]
    logger.debug(f"去掉风速小于切入风速的工况点，剩余数据量：{data3.shape}")
    data3 = data3[data3["风速"] <= velocity_max]
    logger.debug(f"去掉风速大于切出风速的工况点，剩余数据量：{data3.shape}")
    if data3.shape[0] >= 1:
        series = data3.mean()
        if series["有功功率"] < 5:  # 单个风机的功率不会大于5MW，因此如果功率值小于5，则必然单位是MW
            data3["有功功率"] = data3["有功功率"] * 1000000
        elif series["有功功率"] < 5000:  #
            data3["有功功率"] = data3["有功功率"] * 1000
        if series["温度"] < 200:  # 如果环境温度小于200，则单位必然是℃
            data3["温度"] = data3["温度"] + 273.15
        if series["大气压力"] > 800:  # 表明单位为Pa
            pass
        elif series["大气压力"] > 8:  # 表明单位为kPa
            series["大气压力"] = series["大气压力"] * 1000
        else:  # 单位为MPa
            series["大气压力"] = series["大气压力"] * 1000000

    return data3


def split_section(step=1.0, number=10, mode="step"):
    """
    分割风速区间

    :param step: 区间步长
    :param number: 需要分割的区间个数，需要设置mode="number"
    :return:
    """
    result = []
    if mode == "step":
        x_min = velocity_min
        while x_min < velocity_max:
            x_max = x_min + step
            if x_max <= velocity_max:
                result.append([x_min, x_max])
                x_min = x_max
    elif mode == "number":
        pass
    logger.debug(f"风速区间为：{result}")
    return result


def dispatch_data_to_section(data, sections):
    """
    按照风速区间将运行数据分割；并计算每个区间的各项风力发电系统参数

    :param data: 风力发电系统筛选后的运行数据
    :param sections: 区间信息
    :return: 不同风速区间对应的运行数据的列表
    """
    section_data_list = []
    for section in sections:
        data1 = data[(data["风速"] <= section[1]) & (data["风速"] >= section[0])]  # 筛选出风速在区间内的数据
        logger.debug(f"风速在区间{section}中的数据量：{data1.shape[0]}")
        if data1.shape[0] == 0:  # 该风速区间没有数据记录
            section_datum = copy.deepcopy(section_data_list[-1])
            section_datum.velocity_mean = (section[0] + section[1]) / 2
            section_datum.velocity_mean_standard = (section[0] + section[1]) / 2
        else:
            section_datum = SectionData(section, data1)
            section_datum.set_wind_turbine_info(diameter)
            section_datum.set_data_num(data1.shape[0])
            section_datum.calculate_wind_turbine()
            # 如果当前功率比上一个风速区间的功率小，则说明当前区间功率点较少或存在限功率数据，需要将当前区间功率修正到上一个风速区间的功率
            if len(section_data_list) > 0 and section_datum.power_mean < 0.99 * section_data_list[-1].power_mean:
                section_datum.power_mean = section_data_list[-1].power_mean
        section_data_list.append(section_datum)
    return section_data_list


def get_cp_vs_v(section_data_list: List[SectionData]):
    """
    写出每个风机
    :param section_data_list:
    :return:
    """
    velocity = []
    ita = []
    for sect in section_data_list:
        velocity.append(sect.velocity_mean)
        ita.append(sect.ita)
    _data = {"风速": velocity, "功率系数": ita}
    result = pd.DataFrame(_data)
    return result


def get_p_vs_v(section_data_list: List[SectionData]):
    """
    获取功率随风速的变化曲线数据

    :param section_data_list:
    :return:
    """
    if len(velocity_wind_design) == len(power_design) == 0:
        velocity = velocity_wind_design
        power = power_design
    else:
        velocity = []
        power = []
        for sect in section_data_list:
            velocity.append(sect.velocity_mean)
            power.append(sect.power_mean)
    _data = {"风速": velocity, "有功功率": power}
    result = pd.DataFrame(_data)
    return result


def section_data_to_df(section_data_list: List[SectionData]):
    velocity = []
    power = []
    power_kw = []
    cp = []
    num = []
    temperature = []
    power_wind = []
    velocity_standard = []
    power_standard = []
    cp = []
    cp_standard = []
    power_wind_standard = []
    for sect in section_data_list:
        velocity.append(sect.velocity_mean)
        power.append(sect.power_mean)
        power_kw.append(sect.power_mean / 1000)
        cp.append(sect.ita)
        num.append(sect.num_of_data)
        temperature.append(sect.temperature_mean)
        power_wind.append(sect.power_wind)
        velocity_standard.append(sect.velocity_mean_standard)
        power_standard.append(sect.power_mean_standard)
        cp_standard.append(sect.ita_standard)
        power_wind_standard.append(sect.power_wind_standard)
    _data = {"风速": velocity, "有功功率": power, "功率系数": cp, "数据量": num, "风功率": power_wind, "温度": temperature,
             "标准风速": velocity_standard, "标准功率": power_standard, "标准功率kW": power_kw, "标准功率系数": cp_standard,
             "标准风功率": power_wind_standard}
    result = pd.DataFrame(_data)
    return result


def delete_noise_record(raw_data, reference_line, tolerance=0.4, method=None):
    """
    删除噪点数据记录

    :param raw_data:
    :param reference_line:
    :param tolerance:
    :param method: 可以取值"对称剔除"
    :return:
    """
    reference_points = []
    result = []
    for idx, point in reference_line.iterrows():
        reference_points.append((point[0], point[1]))
    for (row_num, series) in raw_data.iterrows():
        v = series["风速"]
        p_item = series["有功功率"]  #
        p_inter = interpolate_value_complex(v, reference_points)
        if method == "对称剔除":
            if abs(p_inter - p_item) / p_inter < tolerance:  # 只有当前测点小于平均值的时候，才可能舍弃测点数据
                result.append(series)
        else:
            if (p_inter - p_item) / p_inter < tolerance:  # 只有当前测点小于平均值的时候，才可能舍弃测点数据
                result.append(series)
            elif v < 10:
                if (p_inter - p_item) / p_inter < tolerance + (1 - v / 10) * (1.3 - tolerance):
                    result.append(series)
    return pd.DataFrame(result)


if __name__ == "__main__":
    for data_file in yield_all_file(data_folder, filter_=[".csv"]):
        base_name = os.path.basename(data_file)
        pure_name = os.path.splitext(base_name)[0]
        xlsx_path = os.path.join(data_folder, f"{pure_name}.xlsx")
        pic_path_ps1 = os.path.join(data_folder, f"{pure_name}_功率散点1.png")
        pic_path_ps2 = os.path.join(data_folder, f"{pure_name}_功率散点2.png")
        pic_path_p = os.path.join(data_folder, f"{pure_name}_功率曲线.png")
        pic_path_p2 = os.path.join(data_folder, f"{pure_name}_功率曲线2.png")
        pic_path_p3 = os.path.join(data_folder, f"{pure_name}_功率曲线3.png")
        pic_path_cp = os.path.join(data_folder, f"{pure_name}_功率系数曲线.png")
        pic_path_cp2 = os.path.join(data_folder, f"{pure_name}_功率系数曲线2.png")

        data = fix_data(data_file)  # 确保存在必要的数据列
        data = filter_data(data)  # 剔除切入、切出风速范围以外和功率为负的数据记录

        # 数据标准化
        data["标准空气密度"] = 1.225
        data["空气气体常数"] = 287.05
        data["rho10"] = data["大气压力"] / data["空气气体常数"] / data["温度"]
        data["标准风速"] = data["风速"] * (data["rho10"] / data["标准空气密度"]) ** (1 / 3)
        data["标准功率"] = data["有功功率"] * data["标准空气密度"] / data["rho10"]

        update_matplotlib_font_config()
        plot_2d_scatter(data["风速"], data["有功功率"] / 1000, show=False, s=10, point_color="#0072BD",
                        x_ticks=x_ticks,
                        x_label="风速（m/s）", y_label="发电机有功功率（kW）", save_to=pic_path_ps1)  # 绘制功率散点图
        sections = split_section(step=0.5)
        section_data = dispatch_data_to_section(data, sections)
        p_vs_v = get_p_vs_v(section_data)
        data_pure = delete_noise_record(raw_data=data, reference_line=p_vs_v, tolerance=0.15)  # 按参考线删除噪点数据
        # data_pure = delete_noise_record(raw_data=data_pure, reference_line=p_vs_v, tolerance=0.3)  # 按参考线删除噪点数据
        # data_pure = delete_noise_record(raw_data=data_pure, reference_line=p_)

        plot_2d_scatter(data_pure["风速"], data_pure["有功功率"] / 1000, show=False, s=10, point_color="#0072BD",
                        x_label="风速（m/s）", y_label="发电机有功功率（kW）", x_ticks=x_ticks, save_to=pic_path_ps2)  # 绘制功率散点图

        section_data = dispatch_data_to_section(data_pure, sections)
        df_sections = section_data_to_df(section_data)
        # plot_2d_line(df_sections["风速"], df_sections["有功功率"] / 1000,
        #              x_label="风速（m/s）", y_label="发电机有功功率（kW）", save_to=pic_path_p)
        # plot_2d_line(df_sections["风速"], df_sections["功率系数"], x_label="风速（m/s）",
        # y_label="功率系数（%）", save_to=pic_path_cp)
        df_sections.sort_values(by=["标准风速", "风速"])
        plot_2d_line(df_sections["风速"], df_sections["有功功率"] / 1000, show=False, x_ticks=x_ticks,
                     x_label="风速（m/s）", y_label="发电机有功功率（kW）", save_to=pic_path_p)
        plot = plot_2d_line(df_sections["风速"], df_sections["有功功率"] / 1000, show=False, line_label="实际功率曲线",
                            x_label="风速（m/s）", y_label="发电机有功功率（kW）", x_ticks=x_ticks)
        plot_2d_line(velocity_wind_design, [p / 1000 for p in power_design], method="append", line_color="blue",
                     plot=plot, show=False,
                     line_label="设计功率曲线", save_to=pic_path_p2, draw_legend=True)
        del plot
        plot = plot_2d_line(df_sections["标准风速"], df_sections["标准功率"] / 1000, show=False, line_label="实际标准功率曲线",
                            x_label="风速（m/s）", y_label="发电机有功功率（kW）", x_ticks=x_ticks, line_color="#41FFA2")
        plot = plot_2d_line(df_sections["风速"], df_sections["有功功率"] / 1000, show=False, line_label="实际功率曲线",
                            plot=plot)
        plot_2d_line(velocity_wind_design, [p / 1000 for p in power_design], method="append", line_color="blue",
                     plot=plot, show=False,
                     line_label="设计功率曲线", save_to=pic_path_p3, draw_legend=True)

        plot_2d_line(df_sections["风速"], df_sections["功率系数"], x_label="风速（m/s）", y_label="功率系数（%）",
                     x_ticks=x_ticks, save_to=pic_path_cp, show=False)
        plot_2d_line(df_sections["风速"], df_sections["标准功率系数"], x_label="风速（m/s）", y_label="标准功率系数（%）",
                     x_ticks=x_ticks, save_to=pic_path_cp2, show=False)
        total_num = df_sections["数据量"].sum()
        sigma_pt = (df_sections["有功功率"] * df_sections["数据量"] / total_num).mean()
        sigma_pw = (df_sections["风功率"] * df_sections["数据量"] / total_num).mean()
        ita_all_section = sigma_pt / sigma_pw
        y = interpolate_value_complex(9, x_list=list(df_sections["风速"]), y_list=list(df_sections["功率系数"]))
        df_ita = pd.DataFrame({"风频加权有功功率平均值": [sigma_pt], "总风功率平均值": [sigma_pw],
                               "单机整体效率": [ita_all_section], "额定功率系数": [y]})
        data_output = {"剔除后的数据": data_pure, "数据点": df_sections, "单机整体效率": df_ita}
        write_excel(xlsx_path, data_output)
    print("done")



import os.path

from yangke.performance.tools.natural_gas import NaturalGas, logger


def gas_properties(compositions=None, visible=False):
    """
    计算天然气物性。


    :param compositions:
    :param visible: 本函数使用excel计算天然气物性，该参数表示计算过程是否显示Excel界面，默认不显示
    :return:
    """
    if compositions is None:
        compositions = {"CH4": 0.9, "N2": 0.1}
    k_idx = {
        "CH4": 1,
        "N2": 52,
        "CO2": 54,
        "C2H6": 2,
        "C3H8": 3,
        "H2O": 42,
        "H2S": 43,
        "H2": 41,
        "CO": 46,
        "O2": 53,
        "C4H10": 4,
        "C4H10-1": 5,
        "C5H12": 6,
        "C5H12-1": 7,
        "C6H14": 9,
        "C7H16": 14,
        "C8H18": 15,
        "C9H20": 16,
        "C10H22": 17,
        "He": 49,
        "Ar": 51,
        "C2H4": 18,
        "C3H6": 19,
        "C4H8": 20,
    }

    app = xw.App(visible=visible, add_book=False)
    app.display_alerts = False
    app.screen_updating = visible
    wb = app.books.open(os.path.join(os.path.dirname(__file__), "resource/ISO6976.xlsx"))
    sht = wb.sheets['Summary']
    for i in range(9, 69, 1):
        sht.range(f"C{i}").value = 0

    for k, v in compositions.items():
        idx = k_idx.get(k)
        row = idx + 8
        sht.range(f"C")

    app.quit()


if __name__ == "__main__":
    comp = {
        "CH4": 95,
        "N2": 1,
        "CO2": 1,
        "C2H6": 3
    }
    gas_properties(comp)



# import numpy as np
# import globalVar as gv
# import os
# import random
#
# import torch
# import torch.nn as nn
# import dataset.data_reader as dr
# from torch.autograd import Variable
# from torch.utils.data import DataLoader, Dataset, TensorDataset
# import torchtext  # NLP处理数据用
# from torchtext.vocab import Vectors
#
# import matplotlib.pyplot as plt
#
#
# def test1_numpy():
#     """
#     使用numpy构建一个全连接ReLU神经网络，一个隐藏层，没有bias。用来从x预测y，使用L2 Loss。
#         h = W_1*x
#         a = max(0, h)
#         y_hat = W_2*a
#     """
#     N, D_in, H, D_out = 64, 1000, 100, 10
#
#     # 随机创建一些训练数据
#     x = np.random.randn(N, D_in)
#     y = np.random.randn(N, D_out)
#
#     w1 = np.random.randn(D_in, H)
#     w2 = np.random.randn(H, D_out)
#
#     learning_rate = 1e-6
#     for it in range(500):
#         # Forward pass
#         h = x.dot(w1)  # N * H
#         h_relu = np.maximum(h, 0)  # N * H
#         y_pred = h_relu.dot(w2)  # N * D_out
#
#         # compute loss
#         loss = np.square(y_pred - y).sum()
#         print(it, loss)
#
#         # Backward pass
#         # compute the gradient
#         grad_loss_to_y_pred = 2 * (y_pred - y)  # N * D_out
#         grad_y_pred_to_w2 = h_relu.T.dot(grad_loss_to_y_pred)  # H * D_out
#         grad_loss_to_w2 = grad_y_pred_to_w2
#         grad_loss_to_h_relu = grad_loss_to_y_pred.dot(w2.T)  # N * H
#         grad_loss_to_h = grad_loss_to_h_relu.copy()  # N * H
#         grad_loss_to_h[h < 0] = 0  # 激活函数ReLUctant
#         grad_loss_to_w1 = x.T.dot(grad_loss_to_h)  # D_in * H
#
#         # update weights of w1 and w2
#         w1 -= learning_rate * grad_loss_to_w1
#         w2 -= learning_rate * grad_loss_to_w2
#
#
# def test1_torch():
#     """
#     使用pytorch构建一个全连接ReLU神经网络，一个隐藏层，没有bias。用来从x预测y，使用L2 Loss。
#         h = W_1*x
#         a = max(0, h)
#         y_hat = W_2*a
#     """
#     autoGradient = True
#
#     N, D_in, H, D_out = 64, 1000, 100, 10
#
#     # 随机创建一些训练数据
#     x = torch.randn(N, D_in)
#     y = torch.randn(N, D_out)
#
#     w1 = torch.randn(D_in, H, requires_grad=autoGradient)
#     w2 = torch.randn(H, D_out, requires_grad=autoGradient)
#
#     learning_rate = 1e-6
#     for it in range(500):
#         if not autoGradient:
#             # Forward pass
#             h = x.mm(w1)  # N * H
#             h_relu = h.clamp(min=0)  # N * H
#             y_pred = h_relu.mm(w2)  # N * D_out
#
#             # compute loss
#             loss = (y_pred - y).pow(2).sum()
#             print(it, loss.item())
#
#             # Backward pass
#             # compute the gradient
#             grad_loss_to_y_pred = 2 * (y_pred - y)  # N * D_out
#             grad_y_pred_to_w2 = h_relu.t().mm(grad_loss_to_y_pred)  # H * D_out
#             grad_loss_to_w2 = grad_y_pred_to_w2
#             grad_loss_to_h_relu = grad_loss_to_y_pred.mm(w2.t())  # N * H
#             grad_loss_to_h = grad_loss_to_h_relu.clone()  # N * H
#             grad_loss_to_h[h < 0] = 0  # 激活函数ReLUctant
#             grad_loss_to_w1 = x.t().mm(grad_loss_to_h)  # D_in * H
#
#             # update weights of w1 and w2
#             w1 -= learning_rate * grad_loss_to_w1
#             w2 -= learning_rate * grad_loss_to_w2
#         # 以上求导步骤pytorch都可以自动完成
#         else:
#             # Forward pass
#             y_pred = x.mm(w1).clamp(min=0).mm(w2)
#             # compute loss
#             loss = (y_pred - y).pow(2).sum()
#             print(it, loss.item())
#             # Backward pass
#
#             loss.backward()
#             with torch.no_grad():
#                 w1 -= learning_rate * w1.grad
#                 w2 -= learning_rate * w2.grad
#                 w1.grad.zero_()
#                 w2.grad.zero_()
#
#
# def test1_torch_nn():
#     """
#     使用pytorch提供的model进行神经网络拟合
#     :return:
#     """
#     N, D_in, H, D_out = 64, 1000, 100, 10
#
#     # 随机创建一些训练数据
#     x = torch.randn(N, D_in)
#     y = torch.randn(N, D_out)
#
#     model = torch.nn.Sequential(
#         torch.nn.Linear(D_in, H, bias=False),
#         torch.nn.ReLU(),
#         torch.nn.Linear(H, D_out, bias=False)
#     )
#     # 这里显式初始化一下模型的weight，不初始化也可以
#     nn.init.normal_(model[0].weight)
#     nn.init.normal_(model[2].weight)
#
#     loss_fn = nn.MSELoss(reduction='sum')
#
#     learning_rate = 1e-6
#     for it in range(500):
#         # Forward pass
#         y_pred = model(x)
#
#         # compute loss
#         loss = loss_fn(y_pred, y)
#         print(it, loss.item())  # python中，loss.item是方法的地址，loss.item()是方法的返回值，注意小括号别少了
#
#         model.zero_grad()
#
#         # Backward pass
#         loss.backward()
#
#         # update weights of w1 and w2
#         with torch.no_grad():
#             for param in model.parameters():
#                 param -= learning_rate * param.grad
#
#
# def test1_torch_nn_optim():
#     """
#     使用pytorch提供的model进行神经网络拟合
#     优化方法使用optim工具
#     :return:
#     """
#     N, D_in, H, D_out = 64, 1000, 100, 10
#
#     # 随机创建一些训练数据
#     x = torch.randn(N, D_in)
#     y = torch.randn(N, D_out)
#
#     model = torch.nn.Sequential(
#         torch.nn.Linear(D_in, H, bias=False),
#         torch.nn.ReLU(),
#         torch.nn.Linear(H, D_out, bias=False)
#     )
#
#     loss_fn = nn.MSELoss(reduction='sum')
#     learning_rate = 1e-3
#     optimizer = torch.optim.Adam(model.parameters(), lr=learning_rate)
#
#     for it in range(500):
#         # Forward pass
#         y_pred = model(x)
#
#         # compute loss
#         loss = loss_fn(y_pred, y)
#         print(it, loss.item())  # python中，loss.item是方法的地址，loss.item()是方法的返回值，注意小括号别少了
#
#         optimizer.zero_grad()
#         # Backward pass
#         loss.backward()
#         optimizer.step()
#
#
# def test1_torch_module():
#     """
#     使用继承自nn.Module的类定义模型
#     优化方法使用optim工具
#     :return:
#     """
#     N, D_in, H, D_out = 64, 1000, 100, 10
#
#     # 随机创建一些训练数据
#     x = torch.randn(N, D_in)
#     y = torch.randn(N, D_out)
#
#     class TwoLayerNet(torch.nn.Module):
#         def __init__(self, D_in, H, D_out):
#             # define the model architecture
#             super(TwoLayerNet, self).__init__()
#             self.linear1 = torch.nn.Linear(D_in, H, bias=False)
#             self.linear2 = torch.nn.Linear(H, D_out, bias=False)
#
#         def forward(self, x):
#             y_pred = self.linear2(self.linear1(x).clamp(min=0))
#             return y_pred
#
#     model = TwoLayerNet(D_in, H, D_out)
#
#     loss_fn = nn.MSELoss(reduction='sum')
#     learning_rate = 1e-4
#     optimizer = torch.optim.Adam(model.parameters(), lr=learning_rate)
#
#     for it in range(500):
#         # Forward pass
#         y_pred = model(x)
#
#         # compute loss
#         loss = loss_fn(y_pred, y)
#         print(it, loss.item())  # python中，loss.item是方法的地址，loss.item()是方法的返回值，注意小括号别少了
#
#         optimizer.zero_grad()
#         # Backward pass
#         loss.backward()
#         optimizer.step()
#
#
# def test1_torch_lstm_language():
#     """
#     https://www.bilibili.com/video/av82398400?p=3
#     语言模型
#
#     :return:
#     """
#     random.seed(53113)
#     np.random.seed(53113)
#     torch.manual_seed(53113)
#
#     BATCH_SIZE = 8
#     EMBEDDING_SIZE = 100
#     MAX_VOCAB_SIZE = 20000
#
#     TEXT = torchtext.data.Field(lower=True)
#     text_path = os.path.join(gv.dataDirectory, 'text8')
#     train, val, test = torchtext.datasets.LanguageModelingDataset.splits(path=text_path,
#                                                                          train='text8.train.txt',
#                                                                          validation='text8.dev.txt',
#                                                                          test='text8.test.txt',
#                                                                          text_field=TEXT)
#     # 根据train数据生成词汇表:Text.vocab，通过vocab.itos[index]将索引转为单词，通过vocab.stoi.get('单词')将单词转为索引
#     TEXT.build_vocab(train, max_size=MAX_VOCAB_SIZE)
#     VOCAB_SIZE = len(TEXT.vocab)  # VOCAB_SIZE会比MAX_VOCAB_SIZE大2，因为torchtext会自动在词汇表中添加<unk>和<pad>两个词
#     print("vocabulary size:{}".format(VOCAB_SIZE))  # {vocabulary size:50002}
#     train_iter, val_iter, test_iter = torchtext.data.BPTTIterator.splits(
#         (train, val, test), batch_size=BATCH_SIZE, device=-1, bptt_len=32, repeat=False, shuffle=True
#     )  # 猜测bptt_len是bptt算法求历史倒数的限制长度，即最多求前bptt_len个历史时刻的倒数，往前则截断，体现在训练数据上则只需要提供bptt_len长度的数据即可
#
#     # 测试数据集是否正确
#     it = iter(train_iter)  # train_iter为torchtext提供的BPTTIterator返回，本身封装了一些语言模型训练时特有的属性
#     for i in range(5):
#         # batch属性在torchtext中定义，包含text和target两个属性，分别对应输入和输出数据，这些数据都是索引值
#         batch = next(it)
#         # batch.text，类型是Tensor，形状是bptt_len*batch_size，我们取batch中第一个输入输出查看
#         print(" ".join([TEXT.vocab.itos[i] for i in batch.text[:, 0].data]))
#         print()
#         print(" ".join([TEXT.vocab.itos[i] for i in batch.target[:, 0].data]))
#
#     class RNNModel(nn.Module):
#         def __init__(self, rnn_type, vocab_size, input_size, hidden_size, num_layers, dropout=0.5):
#             """
#             该模型包含以下几层：
#             -词嵌入层
#             -一个循环神经网络层（RNN、LSTM、GRU）
#             -一个线性层，从hidden state到输出单词表
#             -一个dropout层，用来做regularization
#             """
#             super(RNNModel, self).__init__()
#             self.drop = nn.Dropout(dropout)
#             # 定义一个encoder方法，将输入向量编码为结合词汇表信息的输入变量，会增加输入向量的维度，
#             # 因为输入向量中，一个数字对应一个单词，而编码结果中，一个单词对应一个长度embedding_dim的向量
#             self.embed = nn.Embedding(num_embeddings=vocab_size, embedding_dim=input_size)
#             if rnn_type in ['LSTM', 'GRU']:
#                 inner_model = getattr(nn, rnn_type)
#                 # 根据RNNBase初始化LSTM模型的隐含层数量及各个参数
#                 self.rnn = nn.LSTM(input_size, hidden_size, num_layers, dropout=dropout)
#                 self.rnn = inner_model(input_size, hidden_size, num_layers, dropout=dropout)
#             else:
#                 try:
#                     nonlinearity = {'RNN_TANH': 'tanh', 'RNN_RELU': 'relu'}[rnn_type]
#                 except KeyError:
#                     raise ValueError("""An invalid option for '--model' was supplied,
#                     options are ['LSTM', 'GRU', 'RNN_TANH', or 'RNN_RELU']""")
#                 self.rnn = nn.rnn(input_size, hidden_size, num_layers, nonlinearity=nonlinearity, dropout=dropout)
#
#             # 将隐含层的输入映射到字典索引；输入：N, *, in_features；输出：N, *, out_features
#             self.linear = nn.Linear(in_features=hidden_size, out_features=vocab_size)
#
#             self.init_weights()  # RNN的参数在调用时自动初始化，这里初始化Embedding层和Linear层
#
#             self.rnn_type = rnn_type
#             self.hidden_size = hidden_size
#             self.num_layers = num_layers
#
#         def init_weights(self):
#             init_range = 0.1
#             self.embed.weight.data.uniform_(-init_range, init_range)
#             self.linear.bias.data.zero_()
#             self.linear.weight.data.uniform_(-init_range, init_range)
#
#         # input: [bptt_len, batch_size], hidden: [num_layers, batch_size, hidden_size]
#         def forward(self, input, hidden):
#             emb = self.drop(self.embed(input))  # emb: [bptt_len, batch_size, input_size]
#             # run层的输入和输出形状是相同的
#             output, hidden = self.rnn(emb, hidden)  # output: [bptt_len, batch_size, input_size]
#
#             output = self.drop(output).view(output.size(0) * output.size(1), output.size(2))
#             # [bptt_len * batch_size, input_size]
#             # 这里其实可以不经过变形，因为self.decoder=nn.Linear接受的是[N, *, input_size]形状的数据，
#             # 即只要最后一维的形状对应即可，但展开的话符合线性层神经网络输入到输出的传统对应关系
#             decoded = self.linear(output)  # 将[*, input_size]映射到[*, output_size]，这里output_size=vocab_size
#             decoded = decoded.view(emb.size(0), emb.size(1), decoded.size(1))
#
#             # decoded: [bptt_len, batch_size, vocab_size], hidden: [num_layers, batch_size, hidden_size]
#             return decoded, hidden
#
#         def init_hidden(self, batch_size, required_grad=True):
#             weight = next(self.parameters())
#             new_weight = weight.new_zeros((self.num_layers, batch_size, self.hidden_size),
#                                           requires_grad=required_grad)
#             if self.rnn_type == 'LSTM':
#                 weight_tuple = (new_weight, new_weight)
#                 return weight_tuple
#             else:
#                 return new_weight
#
#     # 初始化模型
#     model = RNNModel('LSTM', VOCAB_SIZE, input_size=EMBEDDING_SIZE, hidden_size=600, num_layers=2, dropout=0.5)
#     loss_fn = nn.CrossEntropyLoss()
#     learning_rate = 0.001
#     optimizer = torch.optim.Adam(model.parameters(), lr=learning_rate)
#     scheduler = torch.optim.lr_scheduler.ExponentialLR(optimizer, 0.5)
#
#     def evaluate(model: RNNModel, data):
#         model.eval()
#         total_loss = 0.
#         it = iter(data)
#         total_count = 0.
#         with torch.no_grad():
#             hidden = model.init_hidden(BATCH_SIZE, required_grad=False)
#             for i, batch in enumerate(it):
#                 data, target = batch.text, batch.target
#                 hidden = repackage_hidden(hidden)
#                 with torch.no_grad():
#                     output, hidden = model(data, hidden)
#                 loss = loss_fn(output.view(-1, VOCAB_SIZE), target.view(-1))
#                 total_count += np.multiply(*data.size())
#                 total_loss += loss.item() * np.multiply(*data.size())
#         loss = total_loss / total_count
#         model.train()
#         return loss
#
#     def repackage_hidden(h):
#         if isinstance(h, torch.Tensor):
#             return h.detach()
#         else:
#             return tuple(repackage_hidden(v) for v in h)
#
#     GRAD_CLIP = 1.
#     NUM_EPOCHS = 2
#     val_losses = []
#     for epoch in range(NUM_EPOCHS):  # 训练过程是不需要字典的，因为是数字到数字的映射，与字典无关
#         # 设置为训练模式，nn.Module类的方法，pycharm追踪不到，RNNModule类也有'train'参数，修改它的train参数只影响RNNModule本身，其他的神经网络层不会变化
#         model.train()
#         it = iter(train_iter)
#         hidden = model.init_hidden(BATCH_SIZE)  # 初始化一个hidden用于输入
#         for i, batch in enumerate(it):
#             # data: [bptt_len, batch_size], target: [bptt_len, batch_size]
#             data, target = batch.text, batch.target
#             hidden = repackage_hidden(hidden)
#             model.zero_grad()
#
#             # data: [bptt_len, batch_size], hidden: [num_layers, batch_size, hidden_size]
#             output, hidden = model(data, hidden)  # output: [bptt_len, batch_size, vocab_size]
#             loss = loss_fn(output.view(-1, VOCAB_SIZE), target.view(-1))
#             loss.backward()
#             torch.nn.utils.clip_grad_norm_(model.parameters(), GRAD_CLIP)
#             optimizer.step()
#             if i % 1000 == 0:
#                 print("epoch {}, iter {}, loss: {}".format(epoch, i, loss.item()))
#
#             if i % 10000 == 0:
#                 val_loss = evaluate(model, val_iter)
#
#                 if len(val_losses) == 0 or val_loss < min(val_losses):
#                     print("best model, val loss:", val_loss)
#                     torch.save(model.state_dict(), "lm-best.th")
#                 else:
#                     scheduler.step()
#                     optimizer = torch.optim.Adam(model.parameters(), lr=learning_rate)
#                 val_losses.append(val_loss)
#
#
# def test2_lstm_stock(stock_code: str):
#     """
#     使用lstm神经元预测股票。
#
#     参考https://blog.csdn.net/a19990412/article/details/85139058
#
#     :param stock_code: 股票代码
#     :return:
#     """
#     file = os.path.join(gv.dataDirectory, stock_code + '.csv')
#     days = 30
#     trainX, trainY, testX, testY = dr.get_data_days_list(file=file, col_x=['close'], days=days, normalization='z-score',
#                                                          split=0.7)
#     gv.logger.debug(
#         "trainX.shape={}, trainY.shape={}, testX.shape={}, testY.shape={}".format(trainX.shape, trainY.shape,
#                                                                                   testX.shape, testY.shape))
#
#     class Model(torch.nn.Module):
#         def __init__(self, input_size):
#             super(Model, self).__init__()
#             self.rnn = nn.LSTM(
#                 input_size=input_size,
#                 hidden_size=64,
#                 num_layers=1,
#             )
#             self.out = nn.Sequential(
#                 nn.Linear(64, 1)
#             )
#
#         def forward(self, x):
#             r_out, (h_n, h_c) = self.rnn(x, None)
#             out = self.out(r_out)
#             return out
#
#     class TrainSet(Dataset):
#         def __init__(self, tx, ty):
#             self.tx, self.ty = tx.float(), ty.float()
#
#         def __getitem__(self, idx):
#             return self.tx[idx], self.ty[idx]
#
#         def __len__(self):
#             return len(self.ty)
#
#     n = 30
#     LR = 0.0001
#     EPOCH = 100
#     trainset = TrainSet(torch.Tensor(trainX), torch.Tensor(trainY))
#     trainloader = DataLoader(trainset, batch_size=16, shuffle=False)
#
#     input_size = days
#     model = Model(input_size)
#     optimizer = torch.optim.Adam(model.parameters(), lr=LR)
#     loss_func = nn.MSELoss()
#
#     for step in range(EPOCH):
#         for x, y in trainloader:
#             x = torch.squeeze(x, dim=2)
#             output = model(x)
#             loss = loss_func(output, y)
#             optimizer.zero_grad()
#             loss.backward()
#             optimizer.step()
#         print(step, loss)
#         if step % 10:
#             torch.save(model, 'model.pkl')
#     torch.save(model, 'model.pkl')
#
#
# def test3_lstm_stock(stock_code: str):
#     """
#     https://blog.csdn.net/weixin_30343157/article/details/102064861
#
#     :param stock_code:
#     :return:
#     """
#     file = os.path.join(gv.dataDirectory, stock_code + '.csv')
#     days = 30
#     train_x, train_y, testX, testY = dr.get_data_days_list(file=file, col_x=['close'], days=days,
#                                                            normalization='z-score',
#                                                            split=0.7)
#     gv.logger.debug(
#         "trainX.shape={}, trainY.shape={}, testX.shape={}, testY.shape={}".format(train_x.shape, train_y.shape,
#                                                                                   testX.shape, testY.shape))
#     train_x = train_x.reshape(-1, 1, 10)
#
#     # train_y = train_y.reshape(-1, 1, 1)
#     train_x = torch.from_numpy(train_x)
#     train_y = torch.from_numpy(train_y)
#
#     class NET(nn.Module):
#         def __init__(self, input_size=10, hidden_size=40, output_size=2, num_layer=2):
#             super(NET, self).__init__()
#             self.rnn = nn.LSTM(input_size, hidden_size, num_layer)
#             self.out = nn.Linear(hidden_size, output_size)
#
#         def forward(self, x):
#             out, _ = self.rnn(x)
#             out = self.out(out[:, -1, :])
#             return out
#
#     net = NET()
#     optimizer = torch.optim.Adam(net.parameters(), lr=0.08, betas=(0.9, 0.999), eps=1e-08, weight_decay=0)
#     loss_func = torch.nn.CrossEntropyLoss()
#
#     for epoch in range(1000):
#         var_x = Variable(train_x).type(torch.FloatTensor)
#         var_y = Variable(train_y).type(torch.LongTensor)
#         out = net(var_x)
#         loss = loss_func(out, var_y)
#         optimizer.zero_grad()
#         loss.backward()
#         optimizer.step()
#         if (epoch + 1) % 100 == 0:
#             print('Epoch: {}, Loss: {:.5f}'.format(epoch + 1, loss.data.numpy()))
#
#     test_x = testX.reshape(-1, 1, 10)
#     test_x = torch.from_numpy(test_x)
#     var_data = Variable(test_x)
#     pred_test = net(var_data)
#     # pred_test = pred_test.view(-1).data.numpy()
#     pred_test = torch.max(pred_test, 1)[1].data.numpy().squeeze()
#
#     plt.plot(pred_test, 'r', label='prediction')
#     plt.plot(testY, 'b', label='real')
#     plt.legend(loc='best')
#     plt.show()
#     print(pred_test, 'prediction number')
#     print(testY, 'real number')
#     j = 0
#     test_size = len(testY)
#     for i in range(test_size):
#         if (pred_test[i] == testY[i]):
#             j = j + 1;
#     j = j / test_size
#     print('Identification:', j)
#
#
# def test4_lstm_stock(stock_code: str):
#     """
#     https://blog.csdn.net/weixin_43855152/article/details/103412077
#     :param stock_code:
#     :return:
#     """
#     file = os.path.join(gv.dataDirectory, stock_code + '.csv')
#     days = 30
#     train_x, train_y, testX, testY = dr.get_data_days_list(file=file, col_x=['close'], days=days,
#                                                            normalization='z-score',
#                                                            split=0.7)
#
#
# def lstm_stock_use_in_build_rnn_拟合(stock_code: str):
#     """
#     使用torch内置的LSTM层进行股票预测，与lstm_stock_use_in_build_rnn()功能相同，只是为了保证复现，修改了一些随机数的机制
#
#     :param stock_code:
#     :return:
#     """
#     torch.manual_seed(10000)
#     np.random.seed(10000)
#     random.seed(10000)
#
#     file = os.path.join(gv.dataDirectory, stock_code + '.csv')
#     days = 30
#     trainX, trainY, testX, testY = dr.get_data_days_list(file=file, col_x=['close'], days=days,
#                                                          normalization='z-score',
#                                                          split=0.7)
#
#     trainX, trainY = torch.from_numpy(trainX).type(torch.float), torch.from_numpy(trainY).type(torch.float)
#     testX, testY = torch.from_numpy(testX).type(torch.float), torch.from_numpy(testY).type(torch.float)
#
#     batch_size = 8
#     train_dataset = TensorDataset(trainX, trainY)
#     test_dataset = TensorDataset(testX, testY)
#     # 只有在数据量很大的时候使用多核加载才能提高速度，数据量小时反而会降低速度
#     train_loader = DataLoader(dataset=train_dataset, batch_size=batch_size, shuffle=False)  # , num_workers=2)
#     test_loader = DataLoader(dataset=test_dataset, batch_size=batch_size, shuffle=False)  # , num_workers=2)
#
#     class RNNModel(nn.Module):
#         # bptt_len=30，即过去30天的数据，input_size=1即每天取一个数据
#         def __init__(self, bptt_len, input_size, hidden_size, num_layers):
#             super(RNNModel, self).__init__()
#             # 定义模型中的神经网络层
#             self.rnn = nn.LSTM(input_size=input_size, hidden_size=hidden_size, num_layers=num_layers, batch_first=True)
#             # 用于将隐含层的输入映射到输出
#             self.linear = nn.Linear(in_features=hidden_size * bptt_len, out_features=1)
#
#             self.hidden_size = hidden_size
#             self.num_layers = num_layers
#
#         # input: [8, 30, 1], hidden: [batch_size, num_layers, hidden_size] # 这里把batch_size调到第一位
#         def forward(self, input):
#             # rnn层的输入和输出形状是相同的
#             output, _ = self.rnn(input)
#
#             # 需要调用output.contiguous()将Tensor的存储整理为连续存储，否则可能报错，原因可能是多卡存储或其他
#             output = output.contiguous().view(output.size(0), output.size(1) * output.size(2))  # [8, 30, 1]->[8, 30]
#
#             # 这里将lstm层输出的Tensor形状的前两位合并，只有输出的维度是需要传出去，以和labels进行比较计算loss
#             output = self.linear(output)  # 将[*, input_size]映射到[*, output_size], [8, 30]->[8, 1]
#
#             output = output.squeeze(-1)  # [8, 1] -> [8]
#             return output
#
#     def evaluate(model: RNNModel, test_loader):
#         model.eval()
#         total_loss = 0.
#         total_count = 0.
#         with torch.no_grad():
#             for i, (data, labels) in enumerate(test_loader):
#                 output = model(data)
#
#                 loss = loss_fn(output, labels)
#                 total_count += len(data)  # 一共多少数据
#                 total_loss += loss.item() * len(data)  # 总的loss
#         loss = total_loss / total_count
#         model.train()
#         return loss
#
#     input_size = 1  # 输入数据的列数，假如考虑过去30天开盘价和收盘价，则input_size=2, bptt_len=30
#     model = RNNModel(bptt_len=days, input_size=input_size, hidden_size=64, num_layers=2)
#     learning_rate = 0.01
#     optimizer = torch.optim.Adam(model.parameters(), lr=learning_rate)
#     scheduler = torch.optim.lr_scheduler.ExponentialLR(optimizer, 0.8)
#
#     loss_fn = torch.nn.MSELoss()
#
#     EPOCHS = 2
#     train_losses = []
#     val_losses = []  # 记录训练过程中的losses，方便后续绘图
#     flag = 0
#     for epoch in range(EPOCHS):
#         # input: [batch_size, bptt_len, input_size], labels: [batch_size]
#         for i, (data, labels) in enumerate(train_loader):
#             # 输入形状是[8, 30, 1]，即[batch_size, bptt_len, input_size]
#             out = model(data)  # out: [8]
#
#             loss = loss_fn(out, labels)
#             optimizer.zero_grad()
#             loss.backward()
#             # torch.nn.utils.clip_grad_norm_(model.parameters(), 1)  # 防止参数过大
#             optimizer.step()
#             if i % 10 == 0:
#                 val_loss = evaluate(model, test_loader)
#
#                 if len(val_losses) == 0 or val_loss < min(val_losses):
#                     gv.logger.info("epoch {}, step {}, loss {}, test loss {}".format(epoch, i, loss, val_loss))
#                     torch.save(model.state_dict(), "./nn_models/pytorchModel/stock-prediction-lstm.th")
#                 elif loss < min(train_losses) and flag < 3:  # 训练集损失减小，测试集损失增大，继续训练，
#                     flag = flag + 1  # 连续三次如此，则更改学习速率
#                     gv.logger.info("epoch {}, step {}, loss {}, test loss {}".format(epoch, i, loss, val_loss))
#                 else:  # 表示训练数据和测试数据都变差
#                     flag = 0
#                     scheduler.step()  # 会自动把optimizer中的lr修改为更小
#                     real_time_lr = optimizer.param_groups[0]['lr']
#                     gv.logger.info("epoch {}, step {}, loss {}, test loss {}, adjust lr to {}"
#                                    .format(epoch, i, loss, val_loss, real_time_lr))
#
#                 # 记录训练和测试数据集的损失
#                 train_losses.append(loss)
#                 val_losses.append(val_loss)
#
#
# def start_mysql_service():
#     # test1_torch_lstm_language()
#     # test3_lstm_stock(gv.symbol)
#     lstm_stock_use_in_build_rnn_拟合(gv.symbol)  # epoch 1, step 50, loss 0.1707535982131958



import os.path
import traceback

import pandas as pd

from yangke.common.config import logger
from yangke.core import runCMD


class PIDB:
    def __init__(self, ip=None):
        self.ip = ip
        self.PI_API = os.path.abspath(os.path.join(os.path.dirname(__file__), "PIAPI", "net6", "PIAPI.exe"))

    def get_snapshot(self, tags, description=None):
        """
        从PI数据库中读取快照值

        :param tags:
        :param description: 点的描述，如果传入描述，则返回的测点标题是描述，否则测点标题是标签名
        :return:
        """
        try:
            cmd = f"{self.PI_API} getsnapshot \"{tags}\""
            output, err = runCMD(cmd, wait_for_result=True)
            res = output
            res_dict = {}
            for line in res.split("\r"):
                line: str = line.strip()
                if line.startswith("#") or line == "":
                    continue
                tag, value = line.split("=")
                if tag in tags:
                    res_dict.update({tag.strip(): float(value)})
            df = pd.DataFrame(data=res_dict.values(), index=list(res_dict.keys())).T
            df.rename(columns=dict(zip(tags, description)), inplace=True)
            return df
        except:
            traceback.print_exc()

    def write_snapshot(self, tags, values):
        try:
            data_ = dict(zip(tags, values))
            cmd = f"{self.PI_API} writesnapshot \"{data_}\""
            runCMD(cmd)
        except:
            traceback.print_exc()
